#ifndef _FPGA_BINARY_MSG_HEADER_
#define _FPGA_BINARY_MSG_HEADER_
#include<stdint.h>

#define FPGA_CLIENT_PROTOCOL_VERSION  "1.12"        

#pragma pack(push)
#pragma pack(1)

//GMD_Protocol
struct CGMDHeader
{
  uint8_t   mVersion;     //协议版本号     
  uint8_t   mExLen;       //扩展长度，FPGA行情中暂未使用
  uint16_t  mTopicID;     //主题号，上交所行情取值0x0201，深交所取值0x0301
  uint16_t  mMsgID;       //消息号
  uint16_t  mBodyLen;     //消息体长度
  uint32_t  mSeqNo;
};

struct CGMDFieldHeader
{
  uint16_t  mFieldID;     //域ID
  uint16_t  mFieldLen;      //域体长度
};

//深交所
//深交所逐笔成交行情域
struct CSZSETradeMDField
{
  int64_t   mApplSeqNum;
  uint32_t  mChannelNo;
  char      mMDStreamID[4];
  int64_t   mBidApplSeqNum;
  int64_t   mOfferApplSeqNum;
  char      mSecurityID[8];
  int64_t   mLastPx;
  int64_t   mLastQty;
  char      mSecurityIDSource[4];
  char      mExecType[4];
  int64_t   mTransactTime;
};

//深交所逐笔委托行情域
struct CSZSEOrderMDField
{
  int64_t   mApplSeqNum;
  uint32_t  mChannelNo;
  char      mMDStreamID[4];
  char      mSecurityID[8];
  int64_t   mPrice;
  int64_t   mOrderQty;
  int64_t   mTransactTime;
  char      mSecurityIDSource[4];
  char      mSide[2];
  char      mOrdType[2];
};

//深交所指数快照行情域
struct CSZSEIndexMDField
{
  int64_t   mOrigTime;
  uint32_t  mChannelNo;
  char      mMDStreamID[4];
  char      mSecurityID[8];
  char      mSecurityIDSource[8];
  char      mTradingPhaseCode[8];
  int64_t   mPreClosePx;
  int64_t   mNumTrades;
  int64_t   mTotalVolumeTrade;
  int64_t   mTotalValueTrade;
  int64_t   mPreCloseIdx;
  int64_t   mLast;
  int64_t   mOpen;
  int64_t   mHigh;
  int64_t   mLow;
};

struct CSZSEDepthMDField
{
  int64_t   mPrice;
  int64_t   mVolume;
};

//深交所现货快照行情域
struct CSZSEStockMDField
{
  int64_t             mOrigTime;
  uint32_t            mChannelNo;
  char                mMDStreamID[4];
  char                mSecurityID[8];
  char                mSecurityIDSource[8];
  char                mTradingPhaseCode[8];
  int64_t             mPreClosePx;
  int64_t             mNumTrades;
  int64_t             mTotalVolumeTrade;
  int64_t             mTotalValueTrade;
  int64_t             mLast;
  int64_t             mOpen;
  int64_t             mHigh;
  int64_t             mLow;
  int64_t             mUpperLimit;
  int64_t             mLowerLimit;
  CSZSEDepthMDField   mBidDepthMD[10];
  int64_t             mBidWeightedAvgPx;
  int64_t             mBidTotalQty;
  CSZSEDepthMDField   mAskDepthMD[10];
  int64_t             mAskWeightedAvgPx;
  int64_t             mAskTotalQty;
  int64_t             mIOPV;
};

//深交所期权快照行情域
struct CSZSEOptionMDField
{
  int64_t             mOrigTime;
  uint32_t            mChannelNo;
  char                mMDStreamID[4];
  char                mSecurityID[8];
  char                mSecurityIDSource[8];
  char                mTradingPhaseCode[8];
  int64_t             mPreClosePx;
  int64_t             mNumTrades;
  int64_t             mTotalVolumeTrade;
  int64_t             mTotalValueTrade;
  int64_t             mLast;
  int64_t             mOpen;
  int64_t             mHigh;
  int64_t             mLow;
  int64_t             mUpperLimit;
  int64_t             mLowerLimit;
  CSZSEDepthMDField   mBidDepthMD[10];
  int64_t             mBidWeightedAvgPx;
  int64_t             mBidTotalQty;
  CSZSEDepthMDField   mAskDepthMD[10];
  int64_t             mAskWeightedAvgPx;
  int64_t             mAskTotalQty;
  int64_t             mPosition;
};

struct CGMDMsgSZSEIndexMD
{
  CGMDHeader          mGMDHdr;
  CGMDFieldHeader     mGMDFieldHdr;
  CSZSEIndexMDField   mGMDField;
};

struct CGMDMsgSZSETradeMD
{
  CGMDHeader          mGMDHdr;
  CGMDFieldHeader     mGMDFieldHdr;
  CSZSETradeMDField   mGMDField;
};

struct CGMDMsgSZSEOrderMD
{
  CGMDHeader          mGMDHdr;
  CGMDFieldHeader     mGMDFieldHdr;
  CSZSEOrderMDField   mGMDField;
};

struct CGMDMsgSZSEStockMD
{
  CGMDHeader          mGMDHdr;
  CGMDFieldHeader     mGMDFieldHdr;
  CSZSEStockMDField   mGMDField;
};

struct CGMDMsgSZSEOptionMD
{
  CGMDHeader          mGMDHdr;
  CGMDFieldHeader     mGMDFieldHdr;
  CSZSEOptionMDField  mGMDField;
};


//上交所
//指数快照行情域
struct CSSEIndexMDField
{
  int32_t   mUpdateTime;
  int32_t   mUpdateMsec;
  char      mTradingDay[8];
  char      mSecurityID[8];
  int64_t   mPreClose;
  int64_t   mOpen; 
  int64_t   mTurnOver;
  int64_t   mHigh; 
  int64_t   mLow;     
  int64_t   mLast;    
  int64_t   mTradeVolume;
  int64_t   mClose;
};

//逐笔成交行情域
struct CSSETradeMDField
{
  char      mTradingDay[8];
  int32_t   mTradeIndex;
  int32_t   mTradeChannel;
  char      mSecurityID[8];
  int32_t   mUpdateTime;
  int32_t   mUpdateMsec;
  int64_t   mPrice;
  int64_t   mTradeVolume;
  int64_t   mTurnOver;
  int64_t   mTradeBuyNo;
  int64_t   mTradeSellNo;
  char      mTradeBSFlag[8];
  int64_t   mBizIndex;
};

//逐笔委托行情域
struct CSSEOrderMDField
{
  char      mTradingDay[8];
  int32_t   mOrderIndex;
  int32_t   mOrderChannel;
  char      mSecurityID[8];
  int32_t   mUpdateTime;
  int32_t   mUpdateMsec;
  char      mOrderType[8];
  int64_t   mOrderNO;
  int64_t   mOrderPrice;
  int64_t   mBalance;
  char      mOrderBSFlag[8];
  int64_t   mBizIndex;
};

struct CSSEDepthMDField
{
  int32_t   mLevel;
  int32_t   mPrice;
  int64_t   mVolume;
};

//现货快照行情域
struct CSSESnapMDField
{
  char              mTradingDay[8];        
  int32_t           mUpdateTime;         
  int32_t           mUpdateMsec;         
  char              mSecurityID[8];         
  int32_t           mPreClose;           
  int32_t           mOpen;            
  int32_t           mHigh;               
  int32_t           mLow;                
  int32_t           mLast;               
  int32_t           mClose;
  char              mTradingPhaseCode[8]; 
  int64_t           mTradeNumber;        
  int64_t           mTradeVolume;        
  int64_t           mTurnOver;
  int64_t           mTotalBidQty;
  int64_t           mTotalOfferQty;
  int32_t           mWeightedAvgBidPx;
  int32_t           mWeightedAvgOfferPx;
  CSSEDepthMDField  mDepthMDField[20];       
};

struct CSSEOptionDepthMDField
{
  double            mPrice;
  int64_t           mVolume;
};

//期权快照行情域
struct CSSEOptionMDField
{
  char                    mTradingDay[8];
  char                    mSecurityID[16];
  int64_t                 mTradeNumber;
  int64_t                 mTradeVolume;
  double                  mTurnOver;
  double                  mPreClose;
  double                  mPreSettlement;
  CSSEOptionDepthMDField  mDepthMDField[10];
  double                  mLast;
  int32_t                 mUpdateTime;
  int32_t                 mUpdateMsec;
  double                  mOpen;
  double                  mClose;
  double                  mSettlement;
  double                  mHigh;
  double                  mLow;
  int32_t                 mLastUpdateTime;
  int32_t                 mLastUpdateMsec;
};

//新债券的快照行情域
struct CSSENewBondSnapMDField
{
  char              mTradingDay[8];        
  int32_t           mUpdateTime;         
  int32_t           mUpdateMsec;         
  char              mSecurityID[8];         
  int32_t           mPreClose;           
  int32_t           mOpen;            
  int32_t           mHigh;               
  int32_t           mLow;                
  int32_t           mLast;               
  int32_t           mClose;
  char              mInstrumentStatus[8]; 
  int64_t           mTradeNumber;        
  int64_t           mTradeVolume;        
  int64_t           mTurnOver;
  int64_t           mTotalBidQty;
  int64_t           mTotalOfferQty;
  int32_t           mAltWeightedAvgBidPx;
  int32_t           mAltWeightedAvgOfferPx;
  int64_t           mWeightedAvgPx;
  CSSEDepthMDField  mDepthMDField[20];       
};

//新债券的逐笔行情域
struct CSSENewBondTickMDField
{
  char      mTradingDay[8];
  int32_t   mTickIndex;
  int32_t   mTickChannel;
  char      mSecurityID[8];
  int32_t   mTickTime;
  char      mtype[4];
  int64_t   mBuyOrderNo;
  int64_t   mSellOrderNo;
  int64_t   mPrice;
  int64_t   mQty;
  int64_t   mTradeMoney;
  char      mTickBSFlag[8];
};

struct CGMDMsgSSEIndexMD
{
  CGMDHeader        mGMDHdr;
  CGMDFieldHeader   mGMDFieldHdr;
  CSSEIndexMDField  mGMDField;
};

struct CGMDMsgSSETradeMD
{
  CGMDHeader        mGMDHdr;
  CGMDFieldHeader   mGMDFieldHdr;
  CSSETradeMDField  mGMDField;
};

struct CGMDMsgSSEOrderMD
{
  CGMDHeader        mGMDHdr;
  CGMDFieldHeader   mGMDFieldHdr;
  CSSEOrderMDField  mGMDField;
};

struct CGMDMsgSSESnapMD
{
  CGMDHeader        mGMDHdr;
  CGMDFieldHeader   mGMDFieldHdr;
  CSSESnapMDField   mGMDField;
};

struct CGMDMsgSSEOptionMD
{
  CGMDHeader        mGMDHdr;
  CGMDFieldHeader   mGMDFieldHdr;
  CSSEOptionMDField mGMDField;
};

struct CGMDMsgSSENewBondSnapMD
{
  CGMDHeader              mGMDHdr;
  CGMDFieldHeader         mGMDFieldHdr;
  CSSENewBondSnapMDField  mGMDField;
};

struct CGMDMsgSSENewBondTickMD
{
  CGMDHeader              mGMDHdr;
  CGMDFieldHeader         mGMDFieldHdr;
  CSSENewBondTickMDField  mGMDField;
};


#pragma pack(pop)

#endif
