﻿#ifndef __TI_MD_DATA_STRUCT_PUB_H__
#define __TI_MD_DATA_STRUCT_PUB_H__


#pragma pack(push, 1)

/************************************************************/
//MdStreamID行情类别,上交所原始数据意义如下:
//MD001 指数类产品
//MD002 股票（A、B 股）类产品
//MD003 债券类产品
//MD004 基金类产品（含公募REITs）
//MD301 期权类产品
//MD101 国债预发行产品
//MD102 盘后固定价格
//MdType:行情类别,与交易所MdStreamID的对应关系
//MD001:1、MD002:2、MD003:3、MD004:4、MD101:5、MD102:6、MD301:7
/************************************************************/

/* struct for  MDStreamID=MD001、MD002、MD003、MD004、MD101、MD102、MD301 message */
struct Sse_MarketData_L1
{
	char   MdType;// 行情类别
	unsigned int UpdateTime : 32;	// 最新更新时间 HHMMSSsss
	char SecurityID[10];            // 证券代码
	double PreClosePrice;           // 昨收价
	long long NumTrades : 64;       // 成交笔数
	double TotalVolumeTrade;        // 成交数量
	double TotalValueTrade;         // 成交总金额
	double LastPrice;               // 最新价
	double OpenPrice;               // 开盘价
	double HighestPrice;            // 最高价
	double LowestPrice;             // 最低价
	double ClosePrice;				// 今收价
	double SettlePrice;             // 今结算价
	double PreSettlePrice;          // 昨结算价
	double Iopv;               		// IOPV
	double PreIopv;      			// 昨IOPV
	double AuctionPrice;			// 动态参考价
	long long AuctionQty:64;		// 虚拟匹配数量
	long long OptionPosVolume : 64; // 总持仓量
	char TradingPhaseCode[9];       // 产品所处的交易阶段代码
	double BidPrice[5];
	double BidVolume[5];
	double AskPrice[5];
	double AskVolume[5];
};
const int Sse_MarketData_L1_s = sizeof(Sse_MarketData_L1);

//深圳行情快照(L2和L1共享一个结构)
/************************************************************/
//MdStreamID行情类别,深交所原始数据意义如下:
//010 现货（股票，基金，债券等）集中竞价交易快照行情
//020 质押式回购交易快照行情
//030 债券分销快照行情
//040 期权集中竞价交易快照行情
//060 以收盘价交易的盘后定价交易快照行情
//061 以成交量加权平均价交易的盘后定价交易快照行情
//630 港股实时行情
//900 指数快照行情
//910 成交量统计指标快照行情
//MdType:行情类别,与交易所MdStreamID的对应关系
//010:10、020:20、030:30、040:40、060:60、061:61、630:63、900:90、910:91
/************************************************************/
struct Szse_MarketData
{
	char   MdType;// 行情类别
	unsigned int UpdateTime : 32;	// 最新更新时间 HHMMSSsss
	char SecurityID[10];            // 证券代码
	char SecurityIDSource[5];       // 证券代码源. 102=深圳证券交易所. 103=香港交易所
	char TradingPhaseCode[9];       // 产品所处的交易阶段代码
	double PreClosePrice;           // 昨收价
	long long NumTrades : 64;       // 成交笔数(成交量)
	double TotalVolumeTrade;        // 成交总量
	double TotalValueTrade;         // 成交总金额
	double LastPrice;               // 最近成交价
	double OpenPrice;               // 开盘成交价
	double HighestPrice;            // 最高成交价
	double LowestPrice;             // 最低成交价
	double VolumeRate;              // 实时成交量加权平均利率，精确到5位小数(质押式回购产品有效)
	double UDPrice1;                // 升跌1(最近价减去昨收价)
	double UDPrice2;                // 升跌2(最近价减去上一最近价，若刚开出开盘价，则=最近价-昨收价)
	double BuyOrdAvgPrice;          // 买入委托数量加权平均价
	double BuyOrdVolume;            // 买入委托总数量
	double SellOrdAvgPrice;         // 卖出委托数量加权平均价
	double SellOrdVolume;           // 卖出委托总数量
	double PERatio1;                // 股票市盈率一(股票类产品有效)
	double PERatio2;                // 股票市盈率二(股票类产品有效)
	double FundValue;               // 基金净值(基金类产品有效)
	double FundRefValue;            // 基金实时参考净值
	double WarrantPremiumRate;      // 权证溢价率(权证类有效有效)
	double LimitUpPrice;            // 涨停价
	double LimitDownPrice;          // 跌停价
	double OptionPosVolume;         // 期权合约的持仓量(期权类有效)
	double AvgRateBp;               // 质押式回购的加权平均利率涨跌BP(质押式回购产品有效)
	double PreCloseVolAvgRate;      // 质押式回购的昨收盘成交量加权平均利率(质押式回购产品有效)
	double BidPrice[10];
	double BidVolume[10];
	double AskPrice[10];
	double AskVolume[10];
};
const int Szse_MarketData_s = sizeof(Szse_MarketData);

// 深圳逐笔委托行情
/************************************************************/
//MdStreamID行情类别,深交所原始数据意义如下:
//011 现货（股票，基金，债券等）集中竞价交易逐笔行情
//021 质押式回购交易逐笔行情
//041 期权集中竞价交易逐笔行情
//051 协议交易逐笔意向行情
//052 协议交易逐笔定价行情
//071 转融通证券出借逐笔行情
//MdType:行情类别,与交易所MdStreamID的对应关系
//011:11、021:21、041:41、051:51、052:52、071:71
/************************************************************/

struct Szse_OrderMD
{
	char   MdType;// 行情类别
	unsigned int UpdateTime : 32;		// 最新更新时间 HHMMSSsss
	char SecurityID[10];				// 证券代码
	char SecurityIDSource[5];			// 证券代码源. 102=深圳证券交易所; 103=香港交易所
	double OrdPrice;					// 委托价格
	double OrderQty;					// 委托数量
	char Side;							// 买卖方向. 1=买; 2=卖; G=借入; F=出借
	unsigned short ChannelNo : 16;      // 频道代码
	long long ApplSeqNum : 64;          // 消息记录号
	char OrdType;						// 订单类型. 1=市价; 2=限价; U=本方最优
	char ConfirmID[9];					// 定价行情约定号
};
const int Szse_OrderMD_s = sizeof(Szse_OrderMD);

// 深圳逐笔成交行情

/************************************************************/
//MdStreamID行情类别,深交所原始数据意义如下:
//011 现货（股票，基金，债券等）集中竞价交易逐笔行情
//021 质押式回购交易逐笔行情
//041 期权集中竞价交易逐笔行情
//051 协议交易逐笔意向行情
//052 协议交易逐笔定价行情
//071 转融通证券出借逐笔行情
//MdType:行情类别,与交易所MdStreamID的对应关系
//011:11、021:21、041:41、051:51、052:52、071:71
/************************************************************/
struct Szse_TradeMD
{
	char   MdType;// 行情类别
	unsigned int UpdateTime : 32;	// 最新更新时间 HHMMSSsss
	char SecurityID[10];            // 证券代码
	char SecurityIDSource[5];       // 证券代码源. 102=深圳证券交易所; 103=香港交易所
	long long BidApplSeqNum : 64;   // 买方委托索引
	long long  OfferApplSeqNum : 64;// 卖方委托索引
	double LastPrice;               // 成交价格
	double LastQty;                 // 成交数量
	char ExecType;                  // 成交类别. 4=撤销; F=成交
	unsigned short ChannelNo : 16;  // 频道代码
	long long ApplSeqNum : 64;      // 消息记录号
};
const int Szse_TradeMD_s = sizeof(Szse_TradeMD);

#pragma pack(pop)

#endif
