﻿/*****************************************
用普通socket接收上海交易所多播行情的例子
如需接收深交所多播行情、逐笔成交行情、逐笔委托行情，需更改：
1 接收的多播IP地址及端口
2 行情结构体名
********************************************/

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <string.h>
#ifdef WIN32
#include <winsock.h>
#else
#include <arpa/inet.h>
#include "sys/socket.h"
#include "netinet/in.h"
#include "unistd.h"
#include "net/if.h"
#endif

#include "timddatastructpub.h"

//多播网络IP地址
#define MULTIIP "239.20.30.90"
//多播网络端口号
#define MULTIPORT 10908
//接收多播数据的本地网卡ip地址
#define LOCALIP "127.0.0.1"

int main()
{
	int ret;
	struct sockaddr_in	addrMulti;	//多播地址
	struct sockaddr*	addrPtr;	//多播地址指针
	unsigned int		addrLen;	//多播地址长度
	struct ip_mreq 		mReq;

	int mysocket=socket(AF_INET,SOCK_DGRAM,0);
	if(mysocket<0)
	{
		printf("建立socket失败\n");
		return -2;
	}

	int on = 1; //允许程序的多个实例运行在同一台机器上
	if(setsockopt(mysocket,SOL_SOCKET,SO_REUSEADDR,&on,sizeof(on))<0)
	{
		printf("允许程序的多个实例运行在同一台机器上失败\n");
		return -3;
	}
	//对多播地址赋值
	addrMulti.sin_family=AF_INET;
	addrMulti.sin_port=htons(MULTIPORT);//多播端口号
	addrMulti.sin_addr.s_addr=inet_addr(MULTIIP);//多播组的IP

	//绑定地址结构到套接字
	ret=bind(mysocket,(struct sockaddr*)&addrMulti,sizeof(addrMulti));//客户端需要绑定端口，用来接收服务器的数据，得指定接收端口，因为数据先从服务器发送过来的
	if(ret<0)
	{
		printf("绑定多播网络地址失败\n");
		return -4;
	}
	//设置接收超时时间为1秒
	struct timeval timeout = {1,0};
	if(setsockopt(mysocket, SOL_SOCKET, SO_RCVTIMEO,&timeout,sizeof(struct timeval)) < 0)
	{
		printf("设置多播网络超时时间失败\n");
		return -5;
	}

	//客户端加入多播组
	mReq.imr_multiaddr.s_addr=inet_addr(MULTIIP);//多播组的IP
	mReq.imr_interface.s_addr=inet_addr(LOCALIP);
	ret = setsockopt(mysocket,IPPROTO_IP,IP_ADD_MEMBERSHIP,&mReq,sizeof(mReq));
	if(ret<0)
	{
		printf("客户端加入多播组失败\n");
		return -6;
	}
	else
	{
		printf("客户端加入多播组:%s:%d|%s\n",MULTIIP,MULTIPORT,LOCALIP);
	}

	addrPtr = (struct sockaddr*)&addrMulti;
	addrLen = sizeof(addrMulti);

	char buffer[1024] = {0};
	int len = 1024;
	while(1)
	{
		int sz = recvfrom(mysocket,buffer,len,0,addrPtr,&addrLen);
		if(sz > 0)
		{
			// 如果需要接收其他行情，请更改Sse_MarketData_L1_s、Sse_MarketData_L1为对应行情的结构体名
			if(Sse_MarketData_L1_s == sz)
			{
				Sse_MarketData_L1 *mdPtr = (Sse_MarketData_L1*)buffer;
				int wtype=mdPtr->MdType;
				
				std::cout << wtype <<","<<mdPtr->UpdateTime<<","<<mdPtr->SecurityID<<","<<
						mdPtr->PreClosePrice<<","<<mdPtr->NumTrades<<","<<mdPtr->TotalVolumeTrade<<","
						<<mdPtr->TotalValueTrade<<","<<mdPtr->LastPrice<<","<<mdPtr->OpenPrice<<","
						<<mdPtr->HighestPrice<<","<<mdPtr->LowestPrice<<","<<mdPtr->ClosePrice<<","<<
						mdPtr->SettlePrice<<","<<mdPtr->PreSettlePrice<<","<<mdPtr->Iopv<<","<<
						mdPtr->PreIopv<<","<<mdPtr->AuctionPrice<<","<<mdPtr->AuctionQty<<","<<
						mdPtr->OptionPosVolume<<","<<mdPtr->TradingPhaseCode<<","<<
						mdPtr->BidPrice[0]<<","<<mdPtr->BidVolume[0]<<","<<mdPtr->AskPrice[0]<<","<<mdPtr->AskVolume[0]<<","<<
						mdPtr->BidPrice[1]<<","<<mdPtr->BidVolume[1]<<","<<mdPtr->AskPrice[1]<<","<<mdPtr->AskVolume[1]<<","<<
						mdPtr->BidPrice[2]<<","<<mdPtr->BidVolume[2]<<","<<mdPtr->AskPrice[2]<<","<<mdPtr->AskVolume[2]<<","<<
						mdPtr->BidPrice[3]<<","<<mdPtr->BidVolume[3]<<","<<mdPtr->AskPrice[3]<<","<<mdPtr->AskVolume[3]<<","<<
						mdPtr->BidPrice[4]<<","<<mdPtr->BidVolume[4]<<","<<mdPtr->AskPrice[4]<<","<<mdPtr->AskVolume[4] << std::endl;
			}
		}
	}
	//离开多播组
	setsockopt(mysocket,IPPROTO_IP,IP_DROP_MEMBERSHIP,&mReq,sizeof(mReq));
	close(mysocket);
}
