﻿
#pragma once

#include <stdio.h>
#include <stdint.h>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <stdarg.h>
#include <cstring>
#include <time.h>
#include "REMDApiStruct.h"

//Tick trade: UA3201,realtime
#define MSG_TYPE_TICKTRADE          "TD"   //3201 

//market overvier:UA3115,3s/update
#define MSG_TYPE_OVERVIEW           "OV" //3115

//Index: 5s/update
#define MSG_TYPE_INDEX              "IX" //3113

//Snapshot: UA3202,3s/update
//10档行情深度及买卖一上的前50笔委托数量等。
#define MSG_TYPE_SNAP               "SP" //3202
#define MSG_TYPE_ORDER_DETAILS      "OD"
#define MSG_TYPE_SH_ORDER_DETAILS   "SHOD"
#define MSG_TYPE_ORDER_QUEUE        "QU"

#define MSG_TYPE_OPTION             "OP"

static uint64_t TickIndexToChanSeq(const uint64_t& lIndex, int16_t& chanNo) {
    //低6个字节
    chanNo = (lIndex >> (6 * 8)) & 0xFFFF;
    //高2个字节
    return (lIndex & 0xFFFFFFFFFFFF);
}


template <typename Ty>
struct output_header {
    output_header(FILE* fp, const Ty* ptr) : fp_(fp), ptr_(ptr) {
        fprintf(fp_, "%s,", typeid(Ty).name());
    }

    ~output_header() {
        fprintf(fp_, "\n");
    }

    FILE* fp_;
    const Ty* ptr_;
};

template <typename Ty>
struct output_data_header {
    output_data_header(FILE* fp, const Ty* ptr) : fp_(fp), ptr_(ptr) {
        fprintf(fp_, "%s:%s %d",
            typeid(Ty).name(), ptr_->code, ptr_->time);
    }

    ~output_data_header() {
        fprintf(fp_, "\n");
    }

    FILE* fp_;
    const Ty* ptr_;
};

template <>
struct output_data_header<REMD_SecuritySnapShot> {
    using data_type = REMD_SecuritySnapShot;
    output_data_header(FILE* fp, const data_type* ptr) : fp_(fp), ptr_(ptr) {
        fprintf(fp_, "%s:%s %d",
            typeid(data_type).name(), ptr_->code, ptr_->time);
    }

    ~output_data_header() {
        fprintf(fp_, "\n");
    }

    FILE* fp_;
    const data_type* ptr_;
};

static void output_to_file(FILE* fp, const char* name, int32_t val) {
    fprintf(fp, " %d,", val);
}

static void output_to_file(FILE* fp, const char* name, uint32_t val) {
    fprintf(fp, " %u,", val);
}

static void output_to_file(FILE* fp, const char* name, const uint32_t* val, std::size_t c)
{
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, " %u,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, const int32_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, " %d,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, const uint64_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, " %lu,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, const int64_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, " %ld,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, int64_t val) {
    fprintf(fp, " %ld,", val);
}

static void output_to_file(FILE* fp, const char* name, uint64_t val) {
    fprintf(fp, " %lu,", val);
}

static void output_to_file(FILE* fp, const char* name, const char* val) {
    fprintf(fp, " %s:%s,", name, val);
}

static void output_to_file(FILE* fp, const char* name, char val) {
    fprintf(fp, " %c,", val);
}

template <std::size_t i, std::size_t j>
void output_to_file(FILE* fp, const char* name, const char(&val)[i][j]) {
    for (int ii = 0; ii < i; ++ii) {
        const char* p = val[ii];
        if (p == nullptr || strlen(p) == 0) {
            fprintf(fp, ",0...,");
            break;
        }
    }
}

#define OUTPUT_FIELD_DATA(name)    output_to_file(fp, #name, field->name)
#define OUTPUT_FIELD_DATA_ARRAY(name, count)    output_to_file(fp, #name, field->name, count)

static void output(FILE* fp, const char* format, ...) {
    va_list args;
    va_start(args, format);
    vfprintf(fp, format, args);
    vfprintf(stdout, format, args);
    va_end(args);
}

static void output(FILE* fp, const REMD_ConnectResult* field) {
    output_header<REMD_ConnectResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(szAddress);
    OUTPUT_FIELD_DATA(nPort);
    OUTPUT_FIELD_DATA(nConnectResult);
}

static void output(FILE* fp, const REMD_Disconnect* field) {
    output_header<REMD_Disconnect> output_header(fp, field);
    OUTPUT_FIELD_DATA(szAddress);
    OUTPUT_FIELD_DATA(nPort);
}

static void output(FILE* fp, const REMD_LoginResult* field) {
    output_header<REMD_LoginResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(szAddress);
    OUTPUT_FIELD_DATA(nPort);
    OUTPUT_FIELD_DATA(nLoginResult);
    OUTPUT_FIELD_DATA(szInfo);
    OUTPUT_FIELD_DATA(markets);
    //OUTPUT_FIELD_DATA(szMarkets);
}

static void output(FILE* fp, const REMD_CodeTableResult* field) {
    output_header<REMD_CodeTableResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(szInfo);
    OUTPUT_FIELD_DATA(markets);
    //OUTPUT_FIELD_DATA(szMarket);
    OUTPUT_FIELD_DATA_ARRAY(nCodeCount, 8);
    OUTPUT_FIELD_DATA_ARRAY(nCodeDate, 8);
}

static void output(FILE* fp, const REMD_RebuildResult* field) {
    output_header<REMD_RebuildResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(Start);
    OUTPUT_FIELD_DATA(End);
    OUTPUT_FIELD_DATA(Status);
}

static void output(FILE* fp, const REMD_CODEINFO* field) {
    output_header<REMD_CODEINFO> output_header(fp, field);
    OUTPUT_FIELD_DATA(code);
    OUTPUT_FIELD_DATA(market);
    OUTPUT_FIELD_DATA(szRemdCode);
    OUTPUT_FIELD_DATA(szCNName);
    OUTPUT_FIELD_DATA(type);
}

static void output(FILE* fp, const REMD_MSG* msg)
{
    fprintf(fp, "nSvrTm:%d,ConnId:%d,Seq:%lu,type:%d,",
        msg->nServerTime, msg->nConnectId, msg->nSequence, msg->nDataType);
}


//----
static inline void tofile(FILE* fp, int32_t val) {
    fprintf(fp, "%d,", val);
}

static inline void tofile(FILE* fp, uint32_t val) {
    fprintf(fp, "%u,", val);
}

static inline void tofile(FILE* fp, const uint32_t* val, std::size_t c)
{
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%u,", val[i]);
    }
}

static inline void tofile(FILE* fp, const int32_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%d,", val[i]);
    }
}

static inline void tofile(FILE* fp, const uint64_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%lu,", val[i]);
    }
}

static inline void tofile(FILE* fp, const int64_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%ld,", val[i]);
    }
}

static inline void tofile(FILE* fp, int64_t val) {
    fprintf(fp, "%ld,", val);
}

static inline void tofile(FILE* fp, uint64_t val) {
    fprintf(fp, "%lu,", val);
}

static inline void tofile(FILE* fp, const char* val) {
    fprintf(fp, "%s,", val);
}

static inline void tofile(FILE* fp, char val) {
    fprintf(fp, "%c,", val);
}

static inline void tofile(FILE* fp, long long int val) {
    fprintf(fp, "%lld,", val);
}

static inline void tofile(FILE* fp, unsigned long long int val) {
    fprintf(fp, "%uld,", val);
}

static inline void tofile(FILE* fp, int64_t* val, std::size_t c) {
    for (int ii = 0; ii < c; ++ii) {
        if ((val + ii) == nullptr) {
            break;
        }
        fprintf(fp, "lu,", (val + ii));
    }
}

//---------------
static void ToFile(FILE* fp, const REMD_SecuritySnapShot* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_SNAP);
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%c,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        //fprintf(fp, "TrdDay:%d,", field->nTradingDay);
        fprintf(fp, "Tm:%d,", field->time);
        fprintf(fp, "Last:%ld,", field->last);
        fprintf(fp, "Stat:%d,", field->status);
        fprintf(fp, "PrePx:%ld,", field->preClose);
        fprintf(fp, "Open:%ld,", field->open);
        fprintf(fp, "High:%ld,", field->high);
        fprintf(fp, "Low:%ld,", field->low);
        fprintf(fp, "NumTrd:%d,", field->numTrades);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "Tnov:%ld,", field->turnover);
        fprintf(fp, "TtlBVol:%ld,", field->totalBidVol);
        fprintf(fp, "TtlAVol:%ld,", field->totalAskVol);
        fprintf(fp, "WtAvgBPx:%ld,", field->weightedAvgBidPrice);
        fprintf(fp, "WtAvgAPx:%ld,", field->weightedAvgAskPrice);
        fprintf(fp, "IOPV:%d,", field->IOPV);
        fprintf(fp, "YdTM:%d,", field->yieldToMaturity);
        fprintf(fp, "HLmt:%ld,", field->highLimited);
        fprintf(fp, "LLmt:%ld,", field->lowLimited);
        fprintf(fp, "Syl1:%d,", field->nSyl1);
        fprintf(fp, "Syl2:%d,", field->nSyl2);
        fprintf(fp, "SD1:%d,", field->nSD1);
        fprintf(fp, "SD2:%d,", field->nSD2);

        fprintf(fp, "AskP:");
        tofile(fp, field->askPrice, 10);
        fprintf(fp, "AskV:");
        tofile(fp, field->askVol, 10);
        fprintf(fp, "BidP:");
        tofile(fp, field->bidPrice, 10);
        fprintf(fp, "BidV:");
        tofile(fp, field->bidVol, 10);

        //fprintf(fp, "AftPx:%ld,", field->afterPrice);
        //fprintf(fp, "AftVol:%ld,", field->afterVolume);
        //fprintf(fp, "AftTov:%ld,", field->afterTurnover);
        //fprintf(fp, "AftMatch:%ld,", field->afterMatchItems);
        fprintf(fp, "%ld,", field->nReserve);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->time);
        tofile(fp, field->last); //last
        tofile(fp, field->numTrades);
        tofile(fp, field->volume);
        tofile(fp, field->turnover);
        tofile(fp, field->askPrice, 10);
        tofile(fp, field->askVol, 10);
        tofile(fp, field->bidPrice, 10);
        tofile(fp, field->bidVol, 10);
        tofile(fp, field->nReserve);
        //tofile(fp, field->totalBidVol);
        //tofile(fp, field->totalAskVol);
        //tofile(fp, field->weightedAvgBidPrice);
        //tofile(fp, field->weightedAvgAskPrice);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_IndexSnapShot* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_INDEX);
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%c,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        //fprintf(fp, "TrdDay:%d,", field->nTradingDay);
        fprintf(fp, "Tm:%d,", field->time);
        fprintf(fp, "PreCls:%ld,", field->preCloseIndex);
        fprintf(fp, "Open:%ld,", field->openIndex);
        fprintf(fp, "High:%ld,", field->highIndex);
        fprintf(fp, "Low:%ld,", field->lowIndex);
        fprintf(fp, "Last:%ld,", field->lastIndex);
        fprintf(fp, "TtlVol:%ld,", field->totalVolume);
        fprintf(fp, "Tnov:%ld,", field->turnover);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->time);
        tofile(fp, field->lastIndex);
        tofile(fp, field->totalVolume);
        tofile(fp, field->turnover);
        //tofile(fp, field->highIndex);
        //tofile(fp, field->lowIndex);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_OrderQueue* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_ORDER_QUEUE);
    if (printHdr) {
    }
    else {
        //output_data_header<REMD_OrderQueue> output_header(fp, field);
        tofile(fp, field->side);
        tofile(fp, field->price);
        tofile(fp, field->orders);
        tofile(fp, field->ABItems);
        //tofile(fp, field->ABVolume, field->ABItems);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_OrderDetail* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_ORDER_DETAILS);
    int16_t channelNo;
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%c,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        fprintf(fp, "Tm:%d,", field->time);
        fprintf(fp, "Seq:%ld,", TickIndexToChanSeq(field->order, channelNo));
        fprintf(fp, "Chnl:%d,", channelNo);
        fprintf(fp, "Px:%ld,", field->price);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "Side:%c,", field->orderSide);
        fprintf(fp, "Type:%c,", field->orderType);
    }
    else {
        //output_data_header<REMD_OrderDetail> output_header(fp, field);
        tofile(fp, field->code);
        tofile(fp, field->time);
        tofile(fp, field->price);
        tofile(fp, field->volume);
        //tofile(fp, field->order);
        fprintf(fp, "%d,", TickIndexToChanSeq(field->order, channelNo));
        fprintf(fp, "%d,", channelNo);
        tofile(fp, field->orderSide);
        tofile(fp, field->orderType);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_TradeDetail* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_TICKTRADE);
    
    int16_t channelNo;
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%c,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        fprintf(fp, "Tm:%d,", field->time);
        fprintf(fp, "Seq:%ld,", TickIndexToChanSeq(field->index, channelNo));
        fprintf(fp, "Chnl:%d,", channelNo);
        fprintf(fp, "AOrd:%d,", field->askOrder);
        fprintf(fp, "BOrd:%d,", field->bidOrder);
        fprintf(fp, "Px:%ld,", field->price);
        fprintf(fp, "Tnov:%ld,", field->turnover);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "ExeType:%c,", field->execType);
        fprintf(fp, "BS:%c,", field->BSFlag);
        fprintf(fp, "Chnl:%d,", field->channelNo);
        fprintf(fp, "BizIdx:%ld,", field->bizIndex);
    }
    else {
        tofile(fp, field->code);
        //tofile(fp, field->nActionDay);
        tofile(fp, field->time);
        tofile(fp, field->price);
        tofile(fp, field->volume);
        tofile(fp, field->turnover);

        //tofile(fp, field->index);     //index
        fprintf(fp, "%d,", TickIndexToChanSeq(field->index, channelNo));
        fprintf(fp, "%d,", channelNo);

        //tofile(fp, field->askOrder);
        fprintf(fp, "%d,", TickIndexToChanSeq(field->askOrder, channelNo));
        fprintf(fp, "%d,", channelNo); 
        //tofile(fp, field->bidOrder);
        fprintf(fp, "%d,", TickIndexToChanSeq(field->bidOrder, channelNo));
        fprintf(fp, "%d,", channelNo);
        tofile(fp, field->BSFlag);
        tofile(fp, field->execType);
        tofile(fp, field->channelNo);
        tofile(fp, field->bizIndex);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_OptionSnapShot* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_OPTION);

    if (printHdr) {

        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%c,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        //fprintf(fp, "TrdDay:%d,", field->nTradingDay);
        fprintf(fp, "Tm:%d,", field->time);
        fprintf(fp, "Last:%ld,", field->last);
        fprintf(fp, "Status:%d,", field->status);
        fprintf(fp, "PrePx:%ld,", field->preClose);
        fprintf(fp, "Open:%ld,", field->open);
        fprintf(fp, "High:%ld,", field->high);
        fprintf(fp, "Low:%ld,", field->low);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "Tnov:%ld,", field->turnover);
        fprintf(fp, "HLmt:%ld,", field->highLimited);
        fprintf(fp, "LLmt:%ld,", field->lowLimited);
        fprintf(fp, "OpnItrt:%ld,", field->openInterest);
        fprintf(fp, "Close:%ld,", field->close);
        fprintf(fp, "StlPx:%ld,", field->settlePrice);
        //fprintf(fp, "PreStlPx:%ld,", field->preSettlePrice);
        //fprintf(fp, "PreOpnItrt:%ld,", field->preOpenInterest);
        //fprintf(fp, "PreDlt:%ld,", field->preDelta);
        fprintf(fp, "CurtDlt:%d,", field->currDelta);
        fprintf(fp, "AuctPx:%ld,", field->auctioprice);
        fprintf(fp, "AuctQty:%ld,", field->auctionQty);

        fprintf(fp, "AskP:");
        tofile(fp, field->askPrice, 5);
        fprintf(fp, "AskV:");
        tofile(fp, field->askVol, 5);
        fprintf(fp, "BidP:");
        tofile(fp, field->bidPrice, 5);
        fprintf(fp, "BidV:");
        tofile(fp, field->bidVol, 5);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->time);
        tofile(fp, field->last);
        tofile(fp, field->volume);
        tofile(fp, field->turnover);
        tofile(fp, field->openInterest);
        tofile(fp, field->askPrice[0]);
        tofile(fp, field->askVol[0]);
        tofile(fp, field->bidPrice[0]);
        tofile(fp, field->bidVol[0]);
        tofile(fp, field->status);

        //tofile(fp, field->preOpenInterest);
        //tofile(fp, field->preClose);
        //tofile(fp, field->preSettlePrice);
        //tofile(fp, field->open);
        //tofile(fp, field->high);
        //tofile(fp, field->low);
        //tofile(fp, field->close);
        //tofile(fp, field->settlePrice);
        //tofile(fp, field->highLimited);
        //tofile(fp, field->lowLimited);
        //tofile(fp, field->preDelta);
        //tofile(fp, field->currDelta);

        //tofile(fp, field->auctioprice);
        //tofile(fp, field->auctionQty);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_SH_OrderDetail* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_SH_ORDER_DETAILS);
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Tm:%d,", field->time);
        fprintf(fp, "Chnl:%d,", field->channelNo);
        fprintf(fp, "Idx:%d,", field->orderIndex);
        fprintf(fp, "Px:%ld,", field->price);
        fprintf(fp, "Vol:%ld,", field->balance);
        fprintf(fp, "Stat:%c,", field->dataStatus);
        fprintf(fp, "odrNo:%ld,", field->orderNo);
        fprintf(fp, "BS:%c,", field->BSFlag);
        fprintf(fp, "Type:%c,", field->orderType);
        fprintf(fp, "bizIdx:%ld,", field->bizIndex);
    }
    else {
        //     code  time channelNo orderIndex                              
        //ORD,603187,142229680,3,6847791,634200,600,6938715,10890984,A,2,0,
        tofile(fp, field->code);
        tofile(fp, field->time);
        tofile(fp, field->channelNo);
        tofile(fp, field->orderIndex);
        tofile(fp, field->price);
        tofile(fp, field->balance);
        tofile(fp, field->orderNo);
        tofile(fp, field->orderType);
        tofile(fp, field->BSFlag);
        tofile(fp, field->bizIndex);
        tofile(fp, (int)field->dataStatus);
    }
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_ConnectResult* field) {
    output_header<REMD_ConnectResult> output_header(fp, field);
    tofile(fp, field->szAddress);
    tofile(fp, field->nPort);
    tofile(fp, field->nConnectResult);
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_Disconnect* field) {
    output_header<REMD_Disconnect> output_header(fp, field);
    tofile(fp, field->szAddress);
    tofile(fp, field->nPort);
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_LoginResult* field) {
    output_header<REMD_LoginResult> output_header(fp, field);
    tofile(fp, field->szAddress);
    tofile(fp, field->nPort);
    tofile(fp, field->nLoginResult);
    tofile(fp, field->szInfo);
    tofile(fp, field->markets);
    //tofile(fp, field->szMarkets);
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_CodeTableResult* field) {
    output_header<REMD_CodeTableResult> output_header(fp, field);
    tofile(fp, field->szInfo);
    tofile(fp, field->markets);
    //tofile(fp, field->szMarket);
    OUTPUT_FIELD_DATA_ARRAY(nCodeCount, 8);
    OUTPUT_FIELD_DATA_ARRAY(nCodeDate, 8);
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_RebuildResult* field) {
    output_header<REMD_RebuildResult> output_header(fp, field);
    tofile(fp, field->Start);
    tofile(fp, field->End);
    tofile(fp, field->Status);
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_CODEINFO* field) {
    output_header<REMD_CODEINFO> output_header(fp, field);
    tofile(fp, field->code);
    tofile(fp, field->market);
    tofile(fp, field->szRemdCode);
    tofile(fp, field->szCNName);
    tofile(fp, field->type);
    fprintf(fp, "\n");
}

static inline void ToFile(FILE* fp, const REMD_MSG* msg) {
    if (fp) {
        fprintf(fp, "nSvrTm:%d,Seq:%lu,type:%d,",
            msg->nServerTime, msg->nSequence, msg->nDataType);
    }
}
