﻿#ifndef __API_TEST_STRUCT_DATA_TEST_H__
#define __API_TEST_STRUCT_DATA_TEST_H__

#include "test_base.h"
#include "sys/stat.h"
#include <sys/types.h>
#include <dirent.h>

// @Class:   StructDataTest
// @Author:  citicsf
// @Date:    2019/08/25
// @Brief:

///服务器时间戳（HHMMSSmmm）转换为绝对ms
static inline uint64_t ConvertServerTm2MSec(int tm) {
    uint64_t hsec = tm / 10000000 * 3600;
    uint64_t msec = tm % 10000000 / 100000 * 60;
    uint64_t sec = tm % 100000 / 1000;
    return (hsec + msec + sec) * 1000 + tm % 1000;
}

class StructDataTest {
    using this_type = StructDataTest;
public:

    static void RemdOnRspData(REMDHandle h, const REMD_MSG* msg)
    {
        struct timespec ts;
        clock_gettime(CLOCK_REALTIME, &ts);
        const char* pStrLocTm = make_time(ts.tv_sec).data();

        static uint64_t mdCount(0);
        FILE* fptr = *DumpFile();
        if (fptr) {
            fprintf(fptr, "%ld.%ld,%ld,", ts.tv_sec, ts.tv_nsec, msg->nSequence);
        }

        //std::cout << make_time(ts.tv_sec) << ',' << ts.tv_nsec << ',';
        switch (msg->nDataType)
        {
        case REMD_DATA_TRADE_DETAIL:
        {
            REMD_TradeDetail* pTrdData = (REMD_TradeDetail*)(msg->pData);
            mdTime = pTrdData->time;
            ToFile(fptr, pTrdData, *PrintMdHeader());
        }   break;
        case REMD_DATA_ORDER_DETAIL:
        {
            REMD_OrderDetail* pOrdData = (REMD_OrderDetail*)(msg->pData);
            mdTime = pOrdData->time;
            ToFile(fptr, pOrdData, *PrintMdHeader());
        }   break;
        case REMD_DATA_SH_ORDER_DETAIL:
        {
            REMD_SH_OrderDetail* pShOrdData = (REMD_SH_OrderDetail*)(msg->pData);
            mdTime = pShOrdData->time;
            ToFile(fptr, pShOrdData, *PrintMdHeader());
        }   break;
        case REMD_DATA_INDEX:
        {
            REMD_IndexSnapShot* pIdxData = (REMD_IndexSnapShot*)(msg->pData);
            mdTime = pIdxData->time;
            ToFile(fptr, pIdxData, *PrintMdHeader());
        }break;
        case REMD_DATA_SECURITY:
        {
            REMD_SecuritySnapShot* pSnapData = (REMD_SecuritySnapShot*)(msg->pData);
            mdTime = pSnapData->time;
            ToFile(fptr, pSnapData, *PrintMdHeader());
        }   break;
        case REMD_DATA_FUTURE:
        {
            REMD_OptionSnapShot* pFutData = (REMD_OptionSnapShot*)(msg->pData);
            mdTime = pFutData->time;
            ToFile(fptr, pFutData, *PrintMdHeader());
        }   break;
        case REMD_DATA_ORDER_QUEUE:
        {
            REMD_OrderQueue* pQueData = (REMD_OrderQueue*)(msg->pData);
            mdTime = pQueData->time;
            ToFile(fptr, pQueData, *PrintMdHeader());
        }   break;
        default:
            //printf("On Invalid DataType: %d\n\n", msg->nDataType);
            return;
        }


        mdCount++;
        if (0 == mdCount % 10000) {
            clock_gettime(CLOCK_REALTIME, &ts);
            printf("%s md +%d\n", pStrLocTm, mdCount);
        }
        fflush(fptr);
    }

    static void get_last_mkt_data(REMDHandle h, const char* szMarket)
    {
        REMD_SecuritySnapShot** pMarketData(NULL);
        int nCount(0);
        std::cout << "---------GetLastMarketData()-----" << std::endl;
        auto errRtn = REMD_GetLastMarketData(h, "SZ", pMarketData, &nCount);
        for (uint32_t i = 0; i < nCount; ++i) {
            auto& mkddt = pMarketData[i];
            std::cout << mkddt->code << ','
                << mkddt->last << ','
                << mkddt->volume << ','
                << mkddt->turnover << std::endl;
        }
        std::cout << std::endl;
        REMD_Free(pMarketData);
    }

    //获取当前最新指数,szMarket(SH,SZ),需调用REMD_Free(void*)释放内存
    static void get_last_index(REMDHandle h, const char* szMarket)
    {
        int nCount(0);
        REMD_IndexSnapShot* pIndexData(nullptr);
        std::cout << "---------REMD_GetLastIndexData()-----" << std::endl;
        REMD_ERROR errRtn = REMD_GetLastIndexData(h, szMarket, &pIndexData, &nCount);
        for (uint32_t i = 0; i < nCount; ++i) {
            auto& indexdt = pIndexData[i];
            std::cout << indexdt.code << ','
                << indexdt.lastIndex << ','
                << indexdt.totalVolume << ','
                << indexdt.turnover << std::endl;
        }
        std::cout << std::endl;
        REMD_Free(pIndexData);
    }

    static void dump_code_table(REMDHandle h, const char* szMarket) {
        FILE* fptr = *CodeTableFile();

        REMD_CODEINFO* pCodeList = nullptr;
        uint32_t nCount = 0;
        auto err = REMD_GetCodeTable(h, szMarket, &pCodeList, &nCount);
        if (err == REMD_ERR_SUCCESS) {
            for (uint32_t i = 0; i < nCount; ++i) {
                auto& code_info_ref = pCodeList[i];
                output(fptr, &code_info_ref);
            }
        }
        else {
            printf("REMD_GetCodeTable::Error: %s\n", REMD_ERROR_DESC(err));
        }
        REMD_Free(pCodeList);
        printf("Market:%s  Count:%u\n", szMarket, nCount);
        printf("dump_code_table(%s) done\n", szMarket);
    }

    static void RemdOnRspSystem(REMDHandle h, const REMD_MSG* msg) {
        assert(h && msg && msg->pData);
        switch (msg->nDataType) {
        case REMD_SYS_UDP_REALTIME_TIMEOUT:
            output<REMD_ConnectResult>(stdout, msg);
            break;
        case REMD_SYS_TCP_RETRANSMIT_CONNECTED:
            output<REMD_ConnectResult>(stdout, msg);
            break;
        case REMD_SYS_TCP_RETRANSMIT_DISCONNECTED:
            output<REMD_Disconnect>(stdout, msg);
            break;
        case REMD_SYS_SERVER_LOGIN:
            output<REMD_LoginResult>(stdout, msg);
            break;
        case REMD_SYS_CODE_TABLE_UPDATED:
            output<REMD_CodeTableResult>(stdout, msg);
            //dump_code_table(h, "SH");
            //dump_code_table(h, "SZ");
            break;
        case REMD_SYS_DATA_REBUILD:
            output<REMD_RebuildResult>(stdout, msg);
            break;
        default:
            printf("On Invalid DataType: %d\n\n", msg->nDataType);
            assert(false);
            break;
        }
    }

    static int Do(const remd::IniParser::Section* section) {
        REMD_OpenSetting open_setting;

        open_setting.ConnectID = 7070;
        std::string value = get_value_and_check(section, "multicast_ip");
        strcpy(open_setting.MulitcastIP, value.c_str());

        value = get_value_and_check(section, "multicast_port");
        open_setting.MulitcastPort = strtoul(value.c_str(), nullptr, 10);

        value = get_value_and_check(section, "multicast_interface");
        strcpy(open_setting.MulitcastInterface, value.c_str());

        value = get_value_and_check(section, "retransmit_ip");
        strcpy(open_setting.RetransmitAddr, value.c_str());

        value = get_value_and_check(section, "retransmit_port");
        open_setting.RetransmitPort = strtoul(value.c_str(), nullptr, 10);

        value = get_value_and_check(section, "user");
        strcpy(open_setting.UserName, value.c_str());

        value = get_value_and_check(section, "passwd");
        strcpy(open_setting.Password, value.c_str());

        value = get_value_and_check(section, "cpu_core");
        open_setting.BindCPU = strtol(value.c_str(), nullptr, 10);
        printf("bind to cpu core:%d\n", open_setting.BindCPU);

        open_setting.DataHandler = RemdOnRspData;
        open_setting.SystemHandler = RemdOnRspSystem;

        std::string markets = section->GetValue("subscribed_market");
        open_setting.SubscribedMarkets = markets.c_str();

#if 1
        value = get_value_and_check(section, "print_header");
        bool* phptr = PrintMdHeader();
        *phptr = stoi(value);

        value = get_value_and_check(section, "file_out");
        bool* foptr = EnableFileOut();
        *foptr = stoi(value);

        //exception detection
        value = get_value_and_check(section, "exception_detect");
        *DetectException() = stoi(value);
        if (*DetectException()) {
            //delay threshold
            value = get_value_and_check(section, "delay_threshold");
            *DelayThresholdMs() = stoi(value);
        }

        if (*EnableFileOut()) {
            std::string sOutFolder("./data/");
            std::string dump(sOutFolder);
            time_t now = time(0);
            tm* ltm = localtime(&now);
            char iyear[50], imonth[50], iday[50], ihour[50], imin[50];
            sprintf(iyear, "%d", 1900 + ltm->tm_year);
            sprintf(imonth, "%02d", 1 + ltm->tm_mon);
            sprintf(iday, "%02d", ltm->tm_mday);
            sprintf(ihour, "%02d", ltm->tm_hour);
            sprintf(imin, "%02d", ltm->tm_min);

            //20180728
            dump += iyear;
            dump += imonth;
            dump += iday;
            dump += ihour;
            dump += imin;
            dump += section->GetValue("dump_file");
            if (!dump.empty()) {
                if (nullptr == opendir(sOutFolder.c_str())) {
                    mkdir(sOutFolder.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
                }

                auto fptr = DumpFile();
                *fptr = fopen(dump.c_str(), "w");
                if (*fptr) {
                    printf("create file: %s.\n", dump.c_str());
                }
                else {
                    printf("create file: %s failed!!\n", dump.c_str());
                }
            }
        }
        else {//print to screen
            *DumpFile() = stdout;
            printf("en_print_file = true.\n");
        }
#else 
        * DumpFile() = stdout;
#endif

        //Get Code table
        InitSymbollistFile();
        std::string codes = section->GetValue("subscribed_codes");
        open_setting.SubscribedCodes = codes.c_str();
        open_setting.SubscribedDataType =
            REMD_DATA_TYPE_ORDER_QUEUE | REMD_DATA_TYPE_ORDER_DETAIL |
            REMD_DATA_TYPE_TRADE_DETAIL;

        REMD_ERROR err;
        auto h = REMD_Open(&open_setting, &err);
        if (h == nullptr) {
            printf("REMD_Open Error: %s\n", REMD_ERROR_DESC(err));
            return -1;
        }
#if 1
        std::this_thread::sleep_for(std::chrono::seconds(2));
        dump_code_table(h, open_setting.SubscribedMarkets);
#endif

#if 0
        std::this_thread::sleep_for(std::chrono::seconds(5));
        REMD_SecuritySnapShot* pp = nullptr;
        int count = 0;
        err = REMD_GetLastMarketData(h, "sz", &pp, &count);
        for (int i = 0; i < count; ++i) {
            auto ref = pp[i];
            output(stdout, &ref);
        }
#endif
        REMD_Wait(h);
        return 0;
    }

    static bool InitSymbollistFile() {
        time_t now = time(0);
        tm* ltm = localtime(&now);
        char iyear[50], imonth[50], iday[50];
        sprintf(iyear, "%d", 1900 + ltm->tm_year);
        sprintf(imonth, "%02d", 1 + ltm->tm_mon);
        sprintf(iday, "%02d", ltm->tm_mday);

        //20180728
        std::string sOutFolder("./instruments/");
        std::string codeFile(sOutFolder);
        codeFile += iyear;
        codeFile += imonth;
        codeFile += iday;
        codeFile += ".txt";
        if (!codeFile.empty()) {
            if (nullptr == opendir(sOutFolder.c_str())) {
                mkdir(sOutFolder.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
            }

            auto fptr = CodeTableFile();
            *fptr = fopen(codeFile.c_str(), "w");
            if (*fptr) {
                printf("create file: %s.\n", codeFile.c_str());
            }
            else {
                printf("create file: %s failed!!\n", codeFile.c_str());
            }
        }
        return true;
    }

    static bool* EnableFileOut() {
        static bool fileout = false;
        return &fileout;
    }

    static FILE** DumpFile() {
        static FILE* fptr = nullptr;
        return &fptr;
    }

    static FILE** CodeTableFile() {
        static FILE* codefptr = nullptr;
        return &codefptr;
    }

    ///异常检测
    static bool* DetectException() {
        static bool detect = false;
        return &detect;
    }

    ///是否打印每个字段的标题
    static bool* PrintMdHeader() {
        static bool printHeader = false;
        return &printHeader;
    }

    ///设置异常检测，包括丢包，延时过大，一段时间内无行情
    static void SetDetectException(bool enable) {
        bool* detect = DetectException();
        *detect = enable;
    }

    ///异常检测延时阈值
    static int32_t* DelayThresholdMs() {
        static int32_t threshold = 0;
        return &threshold;
    }
};

#endif // __API_TEST_STRUCT_DATA_TEST_H__
