
#pragma once

#include <stdio.h>
#include <stdint.h>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <stdarg.h>
#include <cstring>
#include <time.h>
#include "REMDApiStruct.h"

template <typename Ty>
struct output_header {
    output_header(FILE* fp, const Ty* ptr) : fp_(fp), ptr_(ptr) {
        fprintf(fp_, "%s,", typeid(Ty).name());
    }

    ~output_header() {
		//fprintf(fp_, "[%s End]\n", typeid(Ty).name());
		fprintf(fp_, "\n");
    }

    FILE* fp_;
    const Ty* ptr_;
};

template <typename Ty>
struct output_data_header {
    output_data_header(FILE* fp, const Ty* ptr) : fp_(fp), ptr_(ptr) {
        fprintf(fp_, "%s:%s %d",
                typeid(Ty).name(), ptr_->szCode, ptr_->nTime);
    }

    ~output_data_header() {
		//fprintf(fp_, "[%s End]%s\n", typeid(Ty).name(), ptr_->szCode);
		fprintf(fp_, "\n");
    }

    FILE* fp_;
    const Ty* ptr_;
};

template <>
struct output_data_header<REMD_SecuritySnapShot> {
    using data_type = REMD_SecuritySnapShot;
    output_data_header(FILE* fp, const data_type* ptr) : fp_(fp), ptr_(ptr) {
        fprintf(fp_, "%s:%s %d",
                typeid(data_type).name(), ptr_->szCode, ptr_->nTime);
    }

    ~output_data_header() {
		//fprintf(fp_, "[%s End] %s %ld=\n", typeid(data_type).name(), ptr_->szCode, ptr_->nReserve);
		fprintf(fp_, "\n");
    }

    FILE* fp_;
    const data_type* ptr_;
};

static void output_to_file(FILE* fp, const char* name, int32_t val) {
    fprintf(fp, " %s:%d,", name, val);
}

static void output_to_file(FILE* fp, const char* name, uint32_t val) {
    fprintf(fp, " %s:%u,", name, val);
}

static void output_to_file(FILE* fp, const char* name, const uint32_t* val, size_t c) 
{
    fprintf(fp, " %s:", name);
    for (size_t i = 0; i < c; ++i) {
        fprintf(fp, " %u,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, const int32_t* val, size_t c) {
    fprintf(fp, " %s:", name);
    for (size_t i = 0; i < c; ++i) {
        fprintf(fp, " %d,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, const uint64_t* val, size_t c) {
    fprintf(fp, " %s:", name);
    for (size_t i = 0; i < c; ++i) {
        fprintf(fp, " %lu,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, const int64_t* val, size_t c) {
    fprintf(fp, " %s", name);
    for (size_t i = 0; i < c; ++i) {
        fprintf(fp, " %ld,", val[i]);
    }
}

static void output_to_file(FILE* fp, const char* name, int64_t val) {
    fprintf(fp, " %s:%ld,", name, val);
}

static void output_to_file(FILE* fp, const char* name, uint64_t val) {
    fprintf(fp, " %s:%lu,", name, val);
}

static void output_to_file(FILE* fp, const char* name, const char* val) {
    fprintf(fp, " %s:%s,", name, val);
}

static void output_to_file(FILE* fp, const char* name, char val) {
    fprintf(fp, " %s:%c,", name, val);
}

template <size_t i, size_t j>
void output_to_file(FILE* fp, const char* name, const char (& val)[i][j]) {
    fprintf(fp, " %s,", name);
    for (int ii = 0; ii < i; ++ii) {
        const char* p = val[ii];
        if (p == nullptr || strlen(p) == 0) {
            fprintf(fp, " [0...]");
            break;
        }
        fprintf(fp, " [%s]", p);
    }
}

#define OUTPUT_FIELD_DATA(name)    output_to_file(fp, #name, field->name)

#define OUTPUT_FIELD_DATA_ARRAY(name, count)    output_to_file(fp, #name, field->name, count)

static void output(FILE* fp, const char* format, ...) {
    va_list args;
    va_start(args, format);
    vfprintf(fp, format, args);
    vfprintf(stdout, format, args);
    va_end(args);
}

static void output(FILE* fp, const REMD_SecuritySnapShot* field) {
    output_data_header<REMD_SecuritySnapShot> output_header(fp, field);
    OUTPUT_FIELD_DATA(nStatus);
    OUTPUT_FIELD_DATA(nPreClose);
    OUTPUT_FIELD_DATA(nOpen);
    OUTPUT_FIELD_DATA(nHigh);
    OUTPUT_FIELD_DATA(nLow);
    OUTPUT_FIELD_DATA(nMatch);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nAskPrice, 10);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nAskVol, 10);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nBidPrice, 10);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nBidVol, 10);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(nNumTrades);
    OUTPUT_FIELD_DATA(iVolume);
    OUTPUT_FIELD_DATA(iTurnover);
    OUTPUT_FIELD_DATA(nTotalBidVol);
    OUTPUT_FIELD_DATA(nTotalAskVol);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(nWeightedAvgBidPrice);
    OUTPUT_FIELD_DATA(nWeightedAvgAskPrice);
    OUTPUT_FIELD_DATA(nIOPV);
    OUTPUT_FIELD_DATA(nYieldToMaturity);
    OUTPUT_FIELD_DATA(nHighLimited);
    OUTPUT_FIELD_DATA(nLowLimited);
    //fprintf(fp, "\n");
    // it may not safe to printf chPrefix directly
    OUTPUT_FIELD_DATA(nSyl1);
    OUTPUT_FIELD_DATA(nSyl2);
    OUTPUT_FIELD_DATA(nSD2);
}

static void output(FILE* fp, const REMD_IndexSnapShot* field) {
    output_data_header<REMD_IndexSnapShot> output_header(fp, field);
    OUTPUT_FIELD_DATA(nOpenIndex);
    OUTPUT_FIELD_DATA(nHighIndex);
    OUTPUT_FIELD_DATA(nLowIndex);
    OUTPUT_FIELD_DATA(nLastIndex);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(iTotalVolume);
    OUTPUT_FIELD_DATA(iTurnover);
    OUTPUT_FIELD_DATA(nPreCloseIndex);
}

static void output(FILE* fp, const REMD_OrderQueue* field) {
    output_data_header<REMD_OrderQueue> output_header(fp, field);
    OUTPUT_FIELD_DATA(nSide);
    OUTPUT_FIELD_DATA(nPrice);
    OUTPUT_FIELD_DATA(nOrders);
    OUTPUT_FIELD_DATA(nABItems);
    OUTPUT_FIELD_DATA_ARRAY(nABVolume, field->nABItems);
}

static void output(FILE* fp, const REMD_OrderDetail* field) {
    output_data_header<REMD_OrderDetail> output_header(fp, field);
    OUTPUT_FIELD_DATA(nOrder);
    OUTPUT_FIELD_DATA(nPrice);
    OUTPUT_FIELD_DATA(nVolume);
    OUTPUT_FIELD_DATA(nOrderSide);
    OUTPUT_FIELD_DATA(nOrderType);
}

static void output(FILE* fp, const REMD_TradeDetail* field) {
    output_data_header<REMD_TradeDetail> output_header(fp, field);
    OUTPUT_FIELD_DATA(nIndex);
    OUTPUT_FIELD_DATA(nPrice);
    OUTPUT_FIELD_DATA(nVolume);
    OUTPUT_FIELD_DATA(nTurnover);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(nAskOrder);
    OUTPUT_FIELD_DATA(nBidOrder);
    OUTPUT_FIELD_DATA(nExecType);
    OUTPUT_FIELD_DATA(nBSFlag);
}

static void output(FILE* fp, const REMD_FutureSnapShot* field) {
    output_data_header<REMD_FutureSnapShot> output_header(fp, field);
    OUTPUT_FIELD_DATA(nStatus);
    OUTPUT_FIELD_DATA(iPreOpenInterest);
    OUTPUT_FIELD_DATA(nPreClose);
    OUTPUT_FIELD_DATA(nPreSettlePrice);
    OUTPUT_FIELD_DATA(nOpen);
    OUTPUT_FIELD_DATA(nHigh);
    OUTPUT_FIELD_DATA(nLow);
    OUTPUT_FIELD_DATA(nMatch);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(iVolume);
    OUTPUT_FIELD_DATA(iTurnover);
    OUTPUT_FIELD_DATA(iOpenInterest);
    OUTPUT_FIELD_DATA(nClose);
    OUTPUT_FIELD_DATA(nSettlePrice);
   // fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(nHighLimited);
    OUTPUT_FIELD_DATA(nLowLimited);
    OUTPUT_FIELD_DATA(nPreDelta);
    OUTPUT_FIELD_DATA(nCurrDelta);

    OUTPUT_FIELD_DATA_ARRAY(nAskPrice, 5);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nAskVol, 5);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nBidPrice, 5);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA_ARRAY(nBidVol, 5);
    //fprintf(fp, "\n");
    OUTPUT_FIELD_DATA(nAuctionPrice);
    OUTPUT_FIELD_DATA(nAuctionQty);
}

static void output(FILE* fp, const REMD_ConnectResult* field) {
    output_header<REMD_ConnectResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(szAddress);
    OUTPUT_FIELD_DATA(nPort);
    OUTPUT_FIELD_DATA(nConnectResult);
}

static void output(FILE* fp, const REMD_Disconnect* field) {
    output_header<REMD_Disconnect> output_header(fp, field);
    OUTPUT_FIELD_DATA(szAddress);
    OUTPUT_FIELD_DATA(nPort);
}

static void output(FILE* fp, const REMD_LoginResult* field) {
    output_header<REMD_LoginResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(szAddress);
    OUTPUT_FIELD_DATA(nPort);
    OUTPUT_FIELD_DATA(nLoginResult);
    OUTPUT_FIELD_DATA(szInfo);
    OUTPUT_FIELD_DATA(nMarkets);
    OUTPUT_FIELD_DATA(szMarkets);
}

static void output(FILE* fp, const REMD_CodeTableResult* field) {
    output_header<REMD_CodeTableResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(szInfo);
    OUTPUT_FIELD_DATA(nMarkets);
    OUTPUT_FIELD_DATA(szMarket);
    OUTPUT_FIELD_DATA_ARRAY(nCodeCount, 8);
    OUTPUT_FIELD_DATA_ARRAY(nCodeDate, 8);
}

static void output(FILE* fp, const REMD_RebuildResult* field) {
    output_header<REMD_RebuildResult> output_header(fp, field);
    OUTPUT_FIELD_DATA(Start);
    OUTPUT_FIELD_DATA(End);
    OUTPUT_FIELD_DATA(Status);
}

static void output(FILE* fp, const REMD_CODEINFO* field) {
    output_header<REMD_CODEINFO> output_header(fp, field);
    OUTPUT_FIELD_DATA(szCode);
    OUTPUT_FIELD_DATA(nMarket);
    OUTPUT_FIELD_DATA(szRemdCode);
    OUTPUT_FIELD_DATA(szCNName);
    OUTPUT_FIELD_DATA(nType);
}

static void output(FILE* fp, const REMD_MSG* msg) 
{
	fprintf(fp, "nSvrTm:%d,ConnId:%d,Seq:%lu,type:%d,",
		msg->nServerTime, msg->nConnectId, msg->nSequence, msg->nDataType);
}
