#ifndef __API_TEST_STRUCT_DATA_TEST_H__
#define __API_TEST_STRUCT_DATA_TEST_H__

#include "test_base.h"
#include "sys/stat.h"
#include <sys/types.h>
#include <direct.h>
#include <io.h>

// @Class:   StructDataTest
// @Author:  citicsf
// @Date:    2019/08/25
// @Brief:
class StructDataTest {
    using this_type = StructDataTest;
public:

	static void RemdOnRspData(REMDHandle h, const REMD_MSG* msg) 
	{
		struct timespec ts;
		FILE* fptr = NULL;
		
		if (*EnableFileOut()) 
		{
			fptr = *DumpFile(); 
		}

		REMD_SecuritySnapShot* pSSnap(NULL);
		REMD_IndexSnapShot* pIndex(NULL);
		REMD_FutureSnapShot* pFSnap(NULL);
		REMD_TradeDetail* pTrade(NULL);
		REMD_OrderDetail* pODetail(NULL);
		REMD_OrderQueue* pOQueue(NULL);
		assert(h && msg && msg->pData);
		switch (msg->nDataType)
		{
		case REMD_DATA_INDEX:
			if(*EnablePrintOut())
				output<REMD_IndexSnapShot>(stdout, msg);
			if(*EnableFileOut())
				output<REMD_IndexSnapShot>(fptr, msg);
			break;
		case REMD_DATA_SECURITY:
			pSSnap = (REMD_SecuritySnapShot*)(msg->pData);
			if (*EnablePrintOut())
				output<REMD_SecuritySnapShot>(stdout, msg);
			if (*EnableFileOut())
				output<REMD_SecuritySnapShot>(fptr, msg);
			break;
		case REMD_DATA_FUTURE:
			pFSnap = (REMD_FutureSnapShot*)(msg->pData);
			if (*EnablePrintOut())
				output<REMD_FutureSnapShot>(stdout, msg);
			if (*EnableFileOut())
				output<REMD_FutureSnapShot>(fptr, msg);
			break;
		case REMD_DATA_TRADE_DETAIL:
			pTrade = (REMD_TradeDetail*)(msg->pData);
			if (*EnablePrintOut())
				output<REMD_TradeDetail>(stdout, msg);
			if (*EnableFileOut())
				output<REMD_TradeDetail>(fptr, msg);
			break;
		case REMD_DATA_ORDER_DETAIL:
			pODetail = (REMD_OrderDetail*)(msg->pData);
			if (*EnablePrintOut())
				output<REMD_OrderDetail>(stdout, msg);
			if (*EnableFileOut())
				output<REMD_OrderDetail>(fptr, msg);
			break;
		case REMD_DATA_ORDER_QUEUE:
			pOQueue = (REMD_OrderQueue*)(msg->pData);
			if (*EnablePrintOut())
				output<REMD_OrderQueue>(stdout, msg);
			if (*EnableFileOut())
				output<REMD_OrderQueue>(fptr, msg);
			break;
		default:
			printf("On Invalid DataType: %d\n\n", msg->nDataType);
			assert(false);
			break;
		}
    }

	static void get_last_mkt_data(REMDHandle h, const char* szMarket)
	{
		REMD_SecuritySnapShot** pMarketData(NULL);
		int nCount(0);
		std::cout << "---------GetLastMarketData()-----" << std::endl;
		auto errRtn = REMD_GetLastMarketData(h, "SZ", pMarketData, &nCount);
		for (uint32_t i = 0; i < nCount; ++i) {
			auto& mkddt = pMarketData[i];
			std::cout << mkddt->szCode << ','
				<< mkddt->nMatch << ','
				<< mkddt->iVolume << ','
				<< mkddt->iTurnover << std::endl;
		}
		std::cout << std::endl;
		REMD_Free(pMarketData);
	}

	//ȡǰָ,szMarket(SH,SZ),REMD_Free(void*)ͷڴ
	static void get_last_index(REMDHandle h, const char* szMarket)
	{
		int nCount(0);
		REMD_IndexSnapShot* pIndexData(nullptr);
		std::cout << "---------REMD_GetLastIndexData()-----" << std::endl;
		REMD_ERROR errRtn = REMD_GetLastIndexData(h, szMarket, &pIndexData, &nCount);
		for (uint32_t i = 0; i < nCount; ++i) {
			auto& indexdt = pIndexData[i];
			std::cout << indexdt.szCode << ','
				<< indexdt.nLastIndex << ','
				<< indexdt.iTotalVolume << ','
				<< indexdt.iTurnover << std::endl;
		}
		std::cout << std::endl;
		REMD_Free(pIndexData);
	}

    static void dump_code_table(REMDHandle h, const char* szMarket) {
		FILE* fptr = NULL;
		if (*EnableFileOut())
		{
			fptr = *DumpFile();
		}

        REMD_CODEINFO* pCodeList = nullptr;
        uint32_t nCount = 0;
        auto err = REMD_GetCodeTable(h, szMarket, &pCodeList, &nCount);
        if (err == REMD_ERR_SUCCESS) {
			if (*EnableFileOut()) {
				fprintf(fptr, "Market:%s  Count:%u\n", szMarket, nCount);
			}
			for (uint32_t i = 0; i < nCount; ++i) {
				auto& code_info_ref = pCodeList[i];
				output(stdout, &code_info_ref);
				if (*EnableFileOut()){
					output(fptr, &code_info_ref);
				}
            }
        } else {
            printf("REMD_GetCodeTable::Error: %s\n", REMD_ERROR_DESC(err));
        }
        REMD_Free(pCodeList);
    }

    static void RemdOnRspSystem(REMDHandle h, const REMD_MSG* msg) {
		FILE* fptr = NULL;
		if (*EnableFileOut())
		{
			fptr = *DumpFile();
		}
        assert(h && msg && msg->pData);
        switch (msg->nDataType) {
            case REMD_SYS_UDP_REALTIME_TIMEOUT:
                output<REMD_ConnectResult>(stdout, msg);
				if (*EnableFileOut())
					output<REMD_ConnectResult>(fptr, msg);
                break;
            case REMD_SYS_TCP_RETRANSMIT_CONNECTED:
                output<REMD_ConnectResult>(stdout, msg);
				if (*EnableFileOut())
					output<REMD_ConnectResult>(fptr, msg);
                break;
            case REMD_SYS_TCP_RETRANSMIT_DISCONNECTED:
                output<REMD_Disconnect>(stdout, msg);
				if (*EnableFileOut())
					output<REMD_Disconnect>(fptr, msg);
                break;
            case REMD_SYS_SERVER_LOGIN:
                output<REMD_LoginResult>(stdout, msg);
				if (*EnableFileOut())
					output<REMD_LoginResult>(fptr, msg);
                break;
            case REMD_SYS_CODE_TABLE_UPDATED:
                output<REMD_CodeTableResult>(stdout, msg);
				if (*EnableFileOut())
					output<REMD_CodeTableResult>(fptr, msg);
                //dump_code_table(h, "SH");
                //dump_code_table(h, "SZ");
                break;
            case REMD_SYS_DATA_REBUILD:
                output<REMD_RebuildResult>(stdout, msg);
				if (*EnableFileOut())
					output<REMD_RebuildResult>(fptr, msg);
                break;
            default:
                printf("On Invalid DataType: %d\n\n", msg->nDataType);
                assert(false);
                break;
        }
    }

    static int Do(const remd::IniParser::Section* section) {
        REMD_OpenSetting open_setting;

        open_setting.ConnectID = 7070;
        std::string value = get_value_and_check(section, "multicast_ip");
        strcpy(open_setting.MulitcastIP, value.c_str());

        value = get_value_and_check(section, "multicast_port");
        open_setting.MulitcastPort = strtoul(value.c_str(), nullptr, 10);

        value = get_value_and_check(section, "multicast_interface");
        strcpy(open_setting.MulitcastInterface, value.c_str());

        value = get_value_and_check(section, "retransmit_ip");
        strcpy(open_setting.RetransmitAddr, value.c_str());

        value = get_value_and_check(section, "retransmit_port");
        open_setting.RetransmitPort = strtoul(value.c_str(), nullptr, 10);

        value = get_value_and_check(section, "user");
        strcpy(open_setting.UserName, value.c_str());

        value = get_value_and_check(section, "passwd");
        strcpy(open_setting.Password, value.c_str());

        open_setting.DataHandler = RemdOnRspData;
        open_setting.SystemHandler = RemdOnRspSystem;

        std::string markets =  section->GetValue("subscribed_market");
        open_setting.SubscribedMarkets = markets.c_str();

		value = get_value_and_check(section, "file_out");
		bool* foptr = EnableFileOut();
		*foptr = stoi(value);
		value = get_value_and_check(section, "print_out");
		bool* poptr = EnablePrintOut();
		*poptr = stoi(value);

		if (*EnablePrintOut())
		{
			printf("en_print_file = true.\n");
		}
		else
		{
			printf("en_print_file = false.\n");
		}

		if (*EnableFileOut())
		{
			std::string sOutFolder("./data/");
			std::string sDumpFile(sOutFolder);
			time_t now = time(0);
			tm *ltm = localtime(&now);
			char iyear[50], imonth[50], iday[50], ihour[50],imin[50];
			sprintf(iyear, "%d", 1900 + ltm->tm_year);
			sprintf(imonth, "%02d", 1 + ltm->tm_mon);
			sprintf(iday, "%02d", ltm->tm_mday);
			sprintf(ihour, "%02d", ltm->tm_hour);
			sprintf(imin, "%02d", ltm->tm_min);

			//20180728
			sDumpFile += iyear;
			sDumpFile += imonth;
			sDumpFile += iday;
			sDumpFile += ihour;
			sDumpFile += imin;
			sDumpFile += section->GetValue("dump_file");
			if (!sDumpFile.empty()) 
			{
				if (0 != _access(sOutFolder.c_str(), 0))
				{
					_mkdir(sOutFolder.c_str());
				}

				auto fptr = DumpFile();
				*fptr = fopen(sDumpFile.c_str(), "w");
				if (*fptr)
				{
					printf("create file: %s.\n", sDumpFile.c_str());
				}
				else
				{
					printf("create file: %s failed!!\n", sDumpFile.c_str());
				}
			}
		}
		else
		{
			printf("en_write_file = false.\n");
		}

        std::string codes =  section->GetValue( "subscribed_codes");
        open_setting.SubscribedCodes = codes.c_str();
        open_setting.SubscribedDataType =
                REMD_DATA_TYPE_ORDER_QUEUE | REMD_DATA_TYPE_ORDER_DETAIL |
                REMD_DATA_TYPE_TRADE_DETAIL;

        REMD_ERROR err;
        auto h = REMD_Open(&open_setting, &err);
        if (h == nullptr) {
            printf("REMD_Open Error: %s\n", REMD_ERROR_DESC(err));
            return -1;
        }
#if 0
        std::this_thread::sleep_for(std::chrono::seconds(5));
        REMD_SecuritySnapShot* pp = nullptr;
        int count = 0;
        err = REMD_GetLastMarketData(h, "sz", &pp, &count);
        for (int i = 0; i < count; ++i) {
            auto ref = pp[i];
            output(stdout, &ref);
        }
#endif
        REMD_Wait(h);
        return 0;
    }

	static bool* EnableFileOut()
	{
		static bool fileout = false;
		return &fileout;
	}

	static bool* EnablePrintOut()
	{
		static bool printout = false;
		return &printout;
	}

    static FILE** DumpFile() {
        static FILE* fptr = nullptr;
        return &fptr;
    }
};

#endif // __API_TEST_STRUCT_DATA_TEST_H__




