// @Author:   citicsf
// @Date:     2019/08/24
// @Brief:

#ifndef __API_TEST_TEST_BASE_H__
#define __API_TEST_TEST_BASE_H__

#include "REMDApi.h"
#include "output_struct.h"
#include "assert.h"
#include "ini_parser.h"

inline const char* REMD_ERROR_DESC(REMD_ERROR err) {
    switch (err) {
        case REMD_ERR_UNKOWN:
            return "REMD_ERR_UNKOWN";
        case REMD_ERR_INITIALIZE_FAILURE:
            return "REMD_ERR_INITIALIZE_FAILURE";
        case REMD_ERR_NETWORK_ERROR:
            return "REMD_ERR_NETWORK_ERROR";
        case REMD_ERR_INVALID_PARAMS:
            return "REMD_ERR_INVALID_PARAMS";
        case REMD_ERR_VERIFY_FAILURE:
            return "REMD_ERR_VERIFY_FAILURE";
        case REMD_ERR_NO_AUTHORIZED_MARKET:
            return "REMD_ERR_NO_AUTHORIZED_MARKET";
        case REMD_ERR_NO_CODE_TABLE:
            return "REMD_ERR_NO_CODE_TABLE";
        case REMD_ERR_IN_REBUILDING:
            return "REMD_ERR_IN_REBUILDING";
        case REMD_ERR_SUCCESS:
            return "REMD_ERR_SUCCESS";
        default:
            return "REMD_ERR_UNKOWN_UNKOWN";
    }
}

inline const char* REMD_MSG_TYPE_DESC(int type) {
    switch (type) {
        case REMD_MSG_NONE:
            return "REMD_MSG_NONE";
        case REMD_SYS_UDP_REALTIME_TIMEOUT:
            return "REMD_SYS_UDP_REALTIME_TIMEOUT";
        case REMD_SYS_TCP_RETRANSMIT_CONNECTED:
            return "REMD_SYS_TCP_RETRANSMIT_CONNECTED";
        case REMD_SYS_TCP_RETRANSMIT_DISCONNECTED:
            return "REMD_SYS_TCP_RETRANSMIT_DISCONNECTED";
        case REMD_SYS_SERVER_LOGIN:
            return "REMD_SYS_SERVER_LOGIN";
        case REMD_SYS_CODE_TABLE_UPDATED:
            return "REMD_SYS_CODE_TABLE_UPDATED";
        case REMD_SYS_DATA_REBUILD:
            return "REMD_SYS_DATA_REBUILD";
        case REMD_DATA_INDEX:
            return "REMD_DATA_INDEX";
        case REMD_DATA_SECURITY:
            return "REMD_DATA_SECURITY";
        case REMD_DATA_FUTURE:
            return "REMD_DATA_FUTURE";
        case REMD_DATA_TRADE_DETAIL:
            return "REMD_DATA_TRADE_DETAIL";
        case REMD_DATA_ORDER_DETAIL:
            return "REMD_DATA_ORDER_DETAIL";
        case REMD_DATA_ORDER_QUEUE:
            return "REMD_DATA_ORDER_QUEUE";
        case REMD_DATA_EXCHANGE_RAW:
            return "REMD_DATA_EXCHANGE_RAW";
        default:
            return "REMD_MSG_NONE_NONE";
    }
}

template <typename MsgType>
void output(FILE* fptr, const REMD_MSG* msg) {
    if (fptr == nullptr)
        return;
    assert(msg->nDataLen == sizeof(MsgType));
    output(fptr, msg);
    output(fptr, (MsgType*)msg->pData);
}

std::string get_value_and_check(const remd::IniParser::Section* section,
                                const char* key) {
    std::string value = section->GetValue(key);
    if (value.empty()) {
        printf("value empty: %s:%s\n", section->Name(), key);
        throw std::string("bad config");
    }
    return value;
}

std::string make_time(int32_t sec)
{
	char buf[80];
	tm d_tm;
	time_t tvsec = sec;
#if defined( _WIN32 )
	localtime_s(&d_tm, &tvsec);
#else
	localtime_r(&tvsec, &d_tm);
#endif
	size_t chars = ::strftime(buf, 80, "%Y%m%d %H:%M:%S", &d_tm);
	return std::string(buf, chars);
}

inline std::vector<std::string>
StringSplit(const std::string& src, char split) {
    std::vector<std::string> tar;
    auto idx_s = src.begin(), idx_e = src.begin(), str_end = src.end();
    for (; idx_e < str_end; ++idx_e) {
        if (*idx_e == split && idx_e != idx_s) {
            tar.emplace_back(idx_s, idx_e);
            idx_s = idx_e + 1;
        }
    }
    if (idx_e != idx_s) {
        tar.emplace_back(idx_s, idx_e);
    }
    return tar;
}


#endif // __API_TEST_TEST_BASE_H__