﻿
#pragma once

#include <stdio.h>
#include <stdint.h>
#include <string>
#include <cstdio>
#include <typeinfo>
#include <cstring>
#include <time.h>
#include "remd_struct.h"

//Tick trade: UA3201,realtime
#define MSG_TYPE_TICKTRADE          "TD"   //3201 

//market overvier:UA3115,3s/update
#define MSG_TYPE_OVERVIEW           "OV" //3115

//Index: 5s/update
#define MSG_TYPE_INDEX              "IX" //3113

//Snapshot: UA3202,3s/update
//10档行情深度及买卖一上的前50笔委托数量等。
#define MSG_TYPE_SNAP               "SP" //3202
#define MSG_TYPE_ORDER_DETAILS      "OD"
#define MSG_TYPE_SH_ORDER_DETAILS   "SHOD"
#define MSG_TYPE_ORDER_QUEUE        "QU"

#define MSG_TYPE_OPTION             "OP"

static uint64_t TickIndexToChanSeq(const uint64_t& lIndex, int16_t& chanNo) {
    //低6个字节
    chanNo = (lIndex >> (6 * 8)) & 0xFFFF;
    //高2个字节
    return (lIndex & 0xFFFFFFFFFFFF);
}

//----
static inline void tofile(FILE* fp, int32_t val) {
    fprintf(fp, "%d,", val);
}

static inline void tofile(FILE* fp, uint32_t val) {
    fprintf(fp, "%u,", val);
}

static inline void tofile(FILE* fp, const uint32_t* val, std::size_t c)
{
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%u,", val[i]);
    }
}

static inline void tofile(FILE* fp, const int32_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%d,", val[i]);
    }
}

static inline void tofile(FILE* fp, const uint64_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%lu,", val[i]);
    }
}

static inline void tofile(FILE* fp, const int64_t* val, std::size_t c) {
    for (std::size_t i = 0; i < c; ++i) {
        fprintf(fp, "%ld,", val[i]);
    }
}

static inline void tofile(FILE* fp, int64_t val) {
    fprintf(fp, "%ld,", val);
}

static inline void tofile(FILE* fp, uint64_t val) {
    fprintf(fp, "%lu,", val);
}

static inline void tofile(FILE* fp, const char* val) {
    fprintf(fp, "%s,", val);
}

static inline void tofile(FILE* fp, char val) {
    fprintf(fp, "%c,", val);
}

static inline void tofile(FILE* fp, long long int val) {
    fprintf(fp, "%lld,", val);
}

static inline void tofile(FILE* fp, unsigned long long int val) {
    fprintf(fp, "%uld,", val);
}

static inline void tofile(FILE* fp, int64_t* val, std::size_t c) {
    for (int ii = 0; ii < c; ++ii) {
        if ((val + ii) == nullptr) {
            break;
        }
        fprintf(fp, "lu,", (val + ii));
    }
}

//--------remd_struct-------
static void ToFile(FILE* fp, const REMD_SecuritySnapShot* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_SNAP);
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%d,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        //fprintf(fp, "TrdDay:%d,", field->nTradingDay);
        fprintf(fp, "Tm:%ld,", field->timeStamp);
        fprintf(fp, "Last:%ld,", field->last);
        fprintf(fp, "Stat:%d,", field->status);
        fprintf(fp, "PrePx:%ld,", field->preClose);
        fprintf(fp, "Open:%ld,", field->open);
        fprintf(fp, "High:%ld,", field->high);
        fprintf(fp, "Low:%ld,", field->low);
        fprintf(fp, "NumTrd:%d,", field->numTrades);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "Tnov:%ld,", field->turnover);
        fprintf(fp, "TtlBVol:%ld,", field->totalBidVol);
        fprintf(fp, "TtlAVol:%ld,", field->totalAskVol);
        fprintf(fp, "WtAvgBPx:%ld,", field->weightAvgBidPx);
        fprintf(fp, "WtAvgAPx:%ld,", field->weightAvgAskPx);
        fprintf(fp, "IOPV:%d,", field->IOPV);
        fprintf(fp, "YdTM:%d,", field->yieldToMaturity);
        fprintf(fp, "HLmt:%ld,", field->highLimited);
        fprintf(fp, "LLmt:%ld,", field->lowLimited);
        fprintf(fp, "Syl1:%d,", field->Syl1);
        fprintf(fp, "Syl2:%d,", field->Syl2);
        fprintf(fp, "SD1:%d,", field->SD1);
        fprintf(fp, "SD2:%d,", field->SD2);

        fprintf(fp, "AskP:");
        tofile(fp, field->askPx, 10);
        fprintf(fp, "AskV:");
        tofile(fp, field->askVol, 10);
        fprintf(fp, "BidP:");
        tofile(fp, field->bidPx, 10);
        fprintf(fp, "BidV:");
        tofile(fp, field->bidVol, 10);

        //fprintf(fp, "AftPx:%ld,", field->afterPrice);
        //fprintf(fp, "AftVol:%ld,", field->afterVolume);
        //fprintf(fp, "AftTov:%ld,", field->afterTurnover);
        //fprintf(fp, "AftMatch:%ld,", field->afterMatchItems);
        //fprintf(fp, "%ld,", field->nReserve);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->timeStamp);
        tofile(fp, field->last); //last
        tofile(fp, field->numTrades);
        tofile(fp, field->volume);
        tofile(fp, field->turnover);
        tofile(fp, field->askPx, 1);
        tofile(fp, field->askVol, 1);
        tofile(fp, field->bidPx, 1);
        tofile(fp, field->bidVol, 1);
        fprintf(fp, "%d", field->market);
        //tofile(fp, field->totalBidVol);
        //tofile(fp, field->totalAskVol);
        //tofile(fp, field->weightAvgBidPx);
        //tofile(fp, field->weightAvgAskPx);
    }
    fprintf(fp, ";\n");
}

static inline void ToFile(FILE* fp, const REMD_IndexSnapShot* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_INDEX);
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%d,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        //fprintf(fp, "TrdDay:%d,", field->nTradingDay);
        fprintf(fp, "Tm:%ld,", field->timeStamp);
        fprintf(fp, "PreCls:%ld,", field->preCloseIndex);
        fprintf(fp, "Open:%ld,", field->openIndex);
        fprintf(fp, "High:%ld,", field->highIndex);
        fprintf(fp, "Low:%ld,", field->lowIndex);
        fprintf(fp, "Last:%ld,", field->lastIndex);
        fprintf(fp, "TtlVol:%ld,", field->totalVolume);
        fprintf(fp, "Tnov:%ld,", field->turnover);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->timeStamp);
        tofile(fp, field->lastIndex);
        tofile(fp, field->totalVolume);
        tofile(fp, field->turnover);
        fprintf(fp, "%d", field->market);

        //tofile(fp, field->highIndex);
        //tofile(fp, field->lowIndex);
    }
    fprintf(fp, ";\n");
}

static inline void ToFile(FILE* fp, const REMD_OrderQueue* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_ORDER_QUEUE);
    tofile(fp, field->code);
    tofile(fp, field->timeStamp);
    tofile(fp, field->side);
    tofile(fp, field->price);
    tofile(fp, field->orders);
    tofile(fp, field->ABItems);
    //tofile(fp, field->ABVolume, field->ABItems);
    fprintf(fp, ";\n");
}

static inline void ToFile(FILE* fp, const REMD_OrderDetail* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_ORDER_DETAILS);
    int16_t channelNo;
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%d,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        fprintf(fp, "Tm:%ld,", field->timeStamp);
        fprintf(fp, "Seq:%ld,", TickIndexToChanSeq(field->order, channelNo));
        fprintf(fp, "Chnl:%d,", channelNo);
        fprintf(fp, "Px:%ld,", field->price);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "Side:%c,", field->orderSide);
        fprintf(fp, "Type:%c,", field->orderType);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->timeStamp);
        tofile(fp, field->price);
        tofile(fp, field->volume);
        tofile(fp, field->order);
        tofile(fp, field->orderSide);
        tofile(fp, field->orderType);
        fprintf(fp, "%d", field->market);
    }
    fprintf(fp, ";\n");
}

static inline void ToFile(FILE* fp, const REMD_TradeDetail* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_TICKTRADE);
    int16_t channelNo;
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%d,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        fprintf(fp, "Tm:%ld,", field->timeStamp);
        fprintf(fp, "Seq:%ld,", TickIndexToChanSeq(field->tradeIndex, channelNo));
        fprintf(fp, "Chnl:%d,", channelNo);
        fprintf(fp, "AOrd:%d,", field->askOrder);
        fprintf(fp, "BOrd:%d,", field->bidOrder);
        fprintf(fp, "Px:%ld,", field->price);
        fprintf(fp, "Tnov:%ld,", field->turnover);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "ExeType:%c,", field->execType);
        fprintf(fp, "BS:%c,", field->BSFlag);
        fprintf(fp, "Chnl:%d,", field->tradeChannel);
        fprintf(fp, "BizIdx:%ld,", field->bizIndex);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->timeStamp);
        tofile(fp, field->tradeIndex);     //index
        tofile(fp, field->price);
        tofile(fp, field->volume);
        tofile(fp, field->turnover);
        tofile(fp, field->askOrder);
        tofile(fp, field->bidOrder);
        tofile(fp, field->BSFlag);
        tofile(fp, field->execType);
        fprintf(fp, "%d", field->market);
        //tofile(fp, field->tradeChannel);
        //tofile(fp, field->bizIndex);
    }
    fprintf(fp, ";\n");
}

static inline void ToFile(FILE* fp, const REMD_OptionSnapShot* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_OPTION);
    if (printHdr) {

        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Mkt:%d,", field->market);
        //fprintf(fp, "ActDay:%d,", field->nActionDay);
        //fprintf(fp, "TrdDay:%d,", field->nTradingDay);
        fprintf(fp, "Tm:%ld,", field->timeStamp);
        fprintf(fp, "Last:%ld,", field->last);
        fprintf(fp, "Status:%c,", field->status);
        fprintf(fp, "PrePx:%ld,", field->preClose);
        fprintf(fp, "Open:%ld,", field->open);
        fprintf(fp, "High:%ld,", field->high);
        fprintf(fp, "Low:%ld,", field->low);
        fprintf(fp, "Vol:%ld,", field->volume);
        fprintf(fp, "Tnov:%ld,", field->turnover);
        fprintf(fp, "HLmt:%ld,", field->highLimited);
        fprintf(fp, "LLmt:%ld,", field->lowLimited);
        fprintf(fp, "OpnItrt:%ld,", field->openInterest);
        fprintf(fp, "Close:%ld,", field->close);
        fprintf(fp, "StlPx:%ld,", field->settlePrice);
        //fprintf(fp, "PreStlPx:%ld,", field->preSettlePrice);
        //fprintf(fp, "PreOpnItrt:%ld,", field->preOpenInterest);
        //fprintf(fp, "PreDlt:%ld,", field->preDelta);
        fprintf(fp, "CurtDlt:%d,", field->currDelta);
        fprintf(fp, "AuctPx:%ld,", field->auctionPx);
        fprintf(fp, "AuctQty:%ld,", field->auctionQty);

        fprintf(fp, "AskP:");
        tofile(fp, field->askPx, 5);
        fprintf(fp, "AskV:");
        tofile(fp, field->askVol, 5);
        fprintf(fp, "BidP:");
        tofile(fp, field->bidPx, 5);
        fprintf(fp, "BidV:");
        tofile(fp, field->bidVol, 5);
    }
    else {
        tofile(fp, field->code);
        tofile(fp, field->status);
        tofile(fp, field->last);
        tofile(fp, field->volume);
        tofile(fp, field->turnover);
        tofile(fp, field->openInterest);
        tofile(fp, field->askPx[0]);
        tofile(fp, field->askVol[0]);
        tofile(fp, field->bidPx[0]);
        tofile(fp, field->bidVol[0]);
        fprintf(fp, "%d", field->market);

        //tofile(fp, field->preOpenInterest);
        //tofile(fp, field->preClose);
        //tofile(fp, field->preSettlePx);
        //tofile(fp, field->open);
        //tofile(fp, field->high);
        //tofile(fp, field->low);
        //tofile(fp, field->close);
        //tofile(fp, field->settlePrice);
        //tofile(fp, field->highLimited);
        //tofile(fp, field->lowLimited);
        //tofile(fp, field->preDelta);
        //tofile(fp, field->currDelta);

        //tofile(fp, field->auctionPx);
        //tofile(fp, field->auctionQty);
    }
    fprintf(fp, ";\n");
}

///上证逐笔委托(202105新增)
static inline void ToFile(FILE* fp, const REMD_SH_OrderDetail* field, bool printHdr = false) {
    tofile(fp, MSG_TYPE_SH_ORDER_DETAILS);
    if (printHdr) {
        fprintf(fp, "ID:%s,", field->code);
        fprintf(fp, "Tm:%ld,", field->timeStamp);
        fprintf(fp, "Chnl:%d,", field->channelNo);
        fprintf(fp, "Idx:%d,", field->orderIndex);
        fprintf(fp, "Px:%ld,", field->orderPrice);
        fprintf(fp, "Vol:%ld,", field->balance);
        fprintf(fp, "Stat:%c,", field->dataStatus);
        fprintf(fp, "odrNum:%ld,", field->orderNo);
        fprintf(fp, "BS:%c,", field->BSFlag);
        fprintf(fp, "Type:%c,", field->orderType);
        fprintf(fp, "bizIdx:%ld,", field->bizIndex);
    }
    else {//     code     timeStamp   orderIndex channelNo             BSFlag bizIndex 
    //SHOD,603688,20210528142742600,6531049,6,186000,100,6406358,A,2,10091404,0,
        tofile(fp, MSG_TYPE_SH_ORDER_DETAILS);
        tofile(fp, field->code);
        tofile(fp, field->timeStamp);
        tofile(fp, field->channelNo);
        tofile(fp, field->orderIndex);
        tofile(fp, field->orderPrice);
        tofile(fp, field->balance);
        tofile(fp, field->orderNo);
        tofile(fp, field->orderType);
        tofile(fp, field->BSFlag);
        tofile(fp, field->bizIndex);
        tofile(fp, (int)field->dataStatus);
    }
    fprintf(fp, ";\n");
}
