﻿/*****************************************************************************
* @author  : yq
* @date    : $time$
* @file    :
* @brief   :

*****************************************************************************/

#pragma once

#include <unistd.h>
#include <stdint.h>
#include "test_base.h"

///证券原始代码
typedef char REMD_Code[16];
///交易所ID
typedef char REMD_Market;
#define REMD_MARKET_NO  0
#define REMD_MARKET_SH  1
#define REMD_MARKET_SZ  2

///订单类型
typedef char REMD_OrderType;
// 市价
#define REMD_ORDER_TYPE_MARKET  '1'
// 限价
#define REMD_ORDER_TYPE_LIMIT   '2'
// 本方最优
#define REMD_ORDER_TYPE_AT_BEST  'U'

///买卖方向
typedef char REMD_OrderSide;
///买
#define REMD_ORDER_SIDE_BID '1'//对应API的'B'
///卖
#define REMD_ORDER_SIDE_ASK '2'//对应API的'S'
///借入
#define REMD_ORDER_SIDE_G 'G'
///出借
#define REMD_ORDER_SIDE_F 'F'

///成交类型
typedef char REMD_ExecType;
#define REMD_EXEC_TRADE '1'//对应API的'T'
#define REMD_EXEC_CANCEL '2'//对应API的'C'

///BS标记，{外盘、内盘、未知}
typedef char REMD_BSFlag;

#define REMD_BSFLAG_B  '1'//对应API的'B'
#define REMD_BSFLAG_S  '2'//对应API的'S'
#define REMD_BSFLAG_N  ' '

// 市场交易状态定义
enum REMD_MARKET_STATUS {
    Y = 'Y',    //'Y' 新产品.(New Product)
    P = 'P',    //'P' 休市，例如，午餐休市。无撮合市场内部信息披露。(Market Break)
    B = 'B',    //'B' 停牌.(Suspension)
    C = 'C',    //'C' 收市.(Market Closed)
    Z = 'Z',    //'Z' 产品待删除.(Deleted Product)
    V = 'V',    //'V' 结束交易.(End of Trade)
    W = 'W',    //'W' 暂停，除了自有订单和交易的查询之外，任何交易活动都被禁止。(Pause, Trades/Orders Query Only)
    X = 'X',    //'X' 停牌,( 'X'和'W'的区别在于 'X'时可以撤单).(Pause, Allow Trades/Orders Query and Cancelling Order)
    N = 'N',    //'N' 盘中集合竞价订单簿平衡.(Call auction Order Book Balance)
    L = 'L',    //'L' 盘中集合竞价PreOBB.(Call auction PreOBB)
    I = 'I',    //'I' 开市集合竞价.(Market-Open Call auction)
    J = 'J',    //'J' 收盘集合竞价.(Market-Close Call auction)
    U = 'U',    //'U' 盘后处理.(After-Market Process)
    F = 'F',    //'F' 盘前处理.(Pre-Market Process)
    E = 'E',    //'E' 启动.(Start)
    O = 'O',    //'O' 连续撮合.(Continous Matching/Trading)
    Q = 'Q',    //'Q' 连续交易和集合竞价的波动性中断.(Temporarily Circuit Breaker)
    NONE = 0
};

///消息类型定义
enum FID_TYPE {
    ///快照
    SNAP = 0x0101,
    ///指数
    INDEX = 0x0102,
    ///委托队列(上证前50笔订单，目前未使用)
    ORDQUE = 0x0103,
    ///逐笔委托
    ORDDTL = 0x0104,
    ///逐笔成交
    TRDDTL = 0x0105,
    ///快照
    OPTIONS = 0x0106,
    ///上海逐笔委托(202105新增)
    SH_TRDDTL = 0x0107,
};

///-----------------------------------------------------////
#define Tranverse16(X) (((X & 0xff00) >> 8) | \
                        ((X & 0x00ff) << 8))

#define Tranverse32(X) (((X & 0xff000000) >> 24) | \
                        ((X & 0x00ff0000) >> 8) | \
                        ((X & 0x0000ff00) << 8) | \
                        ((X & 0x000000ff) << 24))

#define Tranverse64(X) (((X & 0xff00000000000000) >> 56) | \
                        ((X & 0x00ff000000000000) >> 40) | \
                        ((X & 0x0000ff0000000000) >> 24) | \
                        ((X & 0x000000ff00000000) >> 8) | \
                        ((X & 0x00000000ff000000) << 8) | \
                        ((X & 0x0000000000ff0000) << 24) | \
                        ((X & 0x000000000000ff00) << 40) | \
                        ((X & 0x00000000000000ff) << 56))

//2个字节翻转，0x1256->0x5612
static inline void Tranverse2Byte(void* pDest, void* pSrc)
{
    //*(unsigned int*)pDest = Tranverse16(*(unsigned int*)pSrc);
    *(uint16_t*)pDest = Tranverse16(*(uint16_t*)pSrc);
}

//4个字节翻转，0x12345678->0x78563412
static inline void Tranverse4Byte(void* pDest, void* pSrc)
{
    //*(unsigned int*)pDest = Tranverse32(*(unsigned int*)pSrc);
    *(uint32_t*)pDest = Tranverse32(*(uint32_t*)pSrc);
}

//8个字节翻转，0x123456789abcdef0->f0debc9a78563412
static inline void Tranverse8Byte(void* pDest, void* pSrc)
{
    //*(long long*)pDest = Tranverse64(*(long long*)pSrc);
    *(uint64_t*)pDest = Tranverse64(*(uint64_t*)pSrc);
}

//////////////////////结构定义//////////////////////////
#pragma pack(1)
struct StructFieldHeader {
    //FieldHeader
    uint16_t FID;
    uint16_t Length;  // this field size
    uint64_t Sequence; //
    void EndianSwap() {
        Tranverse2Byte(&FID, &FID);
        Tranverse2Byte(&Length, &Length);
        Tranverse8Byte(&Sequence, &Sequence);
    }
};

struct RemdPacket {
    uint16_t prefixValue;
    uint16_t pktLength;  // this packet size
    uint32_t requestID;
    uint64_t sysTimestamp; // auto gen before send
#if 1
    void EndianSwap() {
        Tranverse2Byte(&prefixValue, &prefixValue);
        Tranverse2Byte(&pktLength, &pktLength);
        Tranverse4Byte(&requestID, &requestID);
        Tranverse8Byte(&sysTimestamp, &sysTimestamp);
    };
#endif
};

///行情快照
struct REMD_SecuritySnapShot {
    REMD_Code   code;                   ///证券代码
    REMD_Market market;                  ///交易所ID
    int64_t     timeStamp;                ///YYYYMMDDHHMMSSsss
    int32_t     status;                  ///当前证券状态，REMD_MARKET_STATUS
    int64_t     preClose;                ///前收盘价
    int64_t     open;                    ///开盘价
    int64_t     high;                    ///最高价
    int64_t     low;                     ///最低价
    int64_t     last;                   ///最新价
    int64_t     askPx[10];            ///申卖价
    int64_t     askVol[10];              ///申卖量
    int64_t     bidPx[10];            ///申买价
    int64_t     bidVol[10];              ///申买量
    int64_t     numTrades;               ///成交笔数
    int64_t     volume;                  ///成交总量
    int64_t     turnover;                ///成交总金额
    int64_t     totalBidVol;             ///委托买入总量
    int64_t     totalAskVol;             ///委托卖出总量
    int64_t     weightAvgBidPx;     ///加权平均委买价格
    int64_t     weightAvgAskPx;     ///加权平均委卖价格
    int64_t     IOPV;                    ///IOPV净值估值
    int64_t     yieldToMaturity;         ///到期收益率
    int64_t     highLimited;             ///涨停价
    int64_t     lowLimited;              ///跌停价
    int64_t     Syl1;                    ///市盈率1
    int64_t     Syl2;                    ///市盈率2
    int64_t     SD1;                     ///升跌1
    int64_t     SD2;                     ///升跌2
    int64_t     afterPx;              ///盘后价格(科创板)
    int64_t     afterVol;             ///盘后量(科创板)
    int64_t     afterTurnover;           ///盘后成交金额(科创板)
    int64_t     afterMatchItems;         ///盘后成交笔数(科创板)
    int64_t     delayTradeSec;            ///
    int64_t     delayTradeNSec;           ///
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse4Byte(&status, &status);
        Tranverse8Byte(&preClose, &preClose);
        Tranverse8Byte(&open, &open);
        Tranverse8Byte(&high, &high);
        Tranverse8Byte(&low, &low);
        Tranverse8Byte(&last, &last);
        for (int i = 0; i < 10; i++) {
            Tranverse8Byte(&askPx[i], &askPx[i]);
            Tranverse8Byte(&askVol[i], &askVol[i]);
            Tranverse8Byte(&bidPx[i], &bidPx[i]);
            Tranverse8Byte(&bidVol[i], &bidVol[i]);
        }
        Tranverse8Byte(&numTrades, &numTrades);
        Tranverse8Byte(&volume, &volume);
        Tranverse8Byte(&turnover, &turnover);
        Tranverse8Byte(&totalBidVol, &totalBidVol);
        Tranverse8Byte(&totalAskVol, &totalAskVol);
        Tranverse8Byte(&weightAvgBidPx, &weightAvgBidPx);
        Tranverse8Byte(&weightAvgAskPx, &weightAvgAskPx);
        Tranverse8Byte(&IOPV, &IOPV);
        Tranverse8Byte(&yieldToMaturity, &yieldToMaturity);
        Tranverse8Byte(&highLimited, &highLimited);
        Tranverse8Byte(&lowLimited, &lowLimited);
        Tranverse8Byte(&Syl1, &Syl1);
        Tranverse8Byte(&Syl2, &Syl2);
        Tranverse8Byte(&SD1, &SD1);
        Tranverse8Byte(&SD2, &SD2);
        Tranverse8Byte(&afterPx, &afterPx);
        Tranverse8Byte(&afterVol, &afterVol);
        Tranverse8Byte(&afterTurnover, &afterTurnover);
        Tranverse8Byte(&afterMatchItems, &afterMatchItems);
        Tranverse8Byte(&delayTradeSec, &delayTradeSec);
        Tranverse8Byte(&delayTradeNSec, &delayTradeNSec);
    };
};


///指数快照行情
struct REMD_IndexSnapShot {
    REMD_Code   code;                       ///证券代码
    REMD_Market market;                 ///交易所ID
    int64_t     timeStamp;                ///YYYYMMDDHHMMSSsss
    int64_t     preCloseIndex;              ///前收盘指数
    int64_t     openIndex;                  ///今开盘指数
    int64_t     highIndex;                  ///最高指数
    int64_t     lowIndex;                   ///最低指数
    int64_t     lastIndex;                 ///最新指数
    int64_t     totalVolume;                ///参与计算相应指数的交易数量
    int64_t     turnover;                   ///参与计算相应指数的成交金额
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse8Byte(&preCloseIndex, &preCloseIndex);
        Tranverse8Byte(&openIndex, &openIndex);
        Tranverse8Byte(&highIndex, &highIndex);
        Tranverse8Byte(&lowIndex, &lowIndex);
        Tranverse8Byte(&lastIndex, &lastIndex);
        Tranverse8Byte(&totalVolume, &totalVolume);
        Tranverse8Byte(&turnover, &turnover);
    }
};

///委托队列
struct REMD_OrderQueue {
    REMD_Code      code;                       ///证券代码
    REMD_Market    market;                      ///交易所ID
    int64_t        timeStamp;                    ///YYYYMMDDHHMMSSsss
    REMD_OrderSide side;                        ///买卖方向
    int64_t        price;                       ///委托价格
    int64_t        orders;                      ///订单数量
    int64_t        ABItems;                     ///委托队列个数
    int64_t        ABVolume[200];               ///委托明细
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse8Byte(&price, &price);
        Tranverse8Byte(&orders, &orders);
        Tranverse8Byte(&ABItems, &ABItems);
        //Tranverse4Byte(&ABVolume[200], &ABVolume);
    }
};

///逐笔委托
struct REMD_OrderDetail {
    REMD_Code      code;                      ///证券代码
    REMD_Market    market;                     ///交易所ID
    int64_t        timeStamp;                   ///YYYYMMDDHHMMSSsss
    uint64_t       order;                      ///委托号
    int64_t        price;                      ///委托价格
    int64_t        volume;                     ///委托数量
    REMD_OrderSide orderSide;                  ///买卖方向
    REMD_OrderType orderType;                  ///订单类型
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse8Byte(&order, &order);
        Tranverse8Byte(&price, &price);
        Tranverse8Byte(&volume, &volume);
    }
};

///逐笔成交
struct REMD_TradeDetail {
    REMD_Code     code;                       ///证券代码
    REMD_Market   market;                     ///交易所ID
    int64_t       timeStamp;                  ///YYYYMMDDHHMMSSsss
    uint64_t      tradeIndex;                 ///成交编号
    int64_t       price;                      ///成交价格
    int64_t       volume;                     ///成交数量
    int64_t       turnover;                   ///成交金额
    uint64_t      askOrder;                   ///卖方委托序号
    uint64_t      bidOrder;                   ///买方委托序号
    REMD_ExecType execType;                   ///成交类型
    REMD_BSFlag   BSFlag;                     ///内外盘类型
    int32_t       tradeChannel;               // 交易通道(上证用，深证为0)
    int64_t       bizIndex;                   // 业务号(上证用，深证该字段消息记录号"ApplSeqNum"，从1开始)
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse8Byte(&tradeIndex, &tradeIndex);
        Tranverse8Byte(&price, &price);
        Tranverse8Byte(&volume, &volume);
        Tranverse8Byte(&turnover, &turnover);
        Tranverse8Byte(&askOrder, &askOrder);
        Tranverse8Byte(&bidOrder, &bidOrder);
        Tranverse4Byte(&tradeChannel, &tradeChannel);
        Tranverse8Byte(&bizIndex, &bizIndex);
    }
};

///期权行情
struct REMD_OptionSnapShot {
    REMD_Code   code;                  ///证券代码
    REMD_Market market;                 ///交易所ID
    int64_t     timeStamp;               ///YYYYMMDDHHMMSSsss
    int32_t     tradingDay;              ///TradingDay
    int32_t     status;                 ///当前证券状态，REMD_MARKET_STATUS
    int64_t     preOpenInterest;        ///昨持仓
    int64_t     preClose;               ///昨收盘价
    int64_t     preSettlePx;            ///昨结算
    int64_t     open;                   ///开盘价
    int64_t     high;                   ///最高价
    int64_t     low;                    ///最低价
    int64_t     last;                  ///最新价
    int64_t     volume;                 ///成交总量
    int64_t     turnover;               ///成交总金额
    int64_t     openInterest;           ///持仓总量
    int64_t     close;                  ///今收盘
    int64_t     settlePrice;            ///今结算
    int64_t     highLimited;            ///涨停价
    int64_t     lowLimited;             ///跌停价
    int32_t     preDelta;               ///昨虚实度
    int32_t     currDelta;              ///今虚实度
    int64_t     askPx[5];               ///申卖价
    int64_t     askVol[5];              ///申卖量
    int64_t     bidPx[5];               ///申买价
    int64_t     bidVol[5];              ///申买量
    int64_t     auctionPx;              ///波动性中断参考价
    int64_t     auctionQty;             ///波动性中断集合竞价虚拟匹配量
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse4Byte(&status, &status);
        Tranverse4Byte(&tradingDay, &tradingDay);
        Tranverse8Byte(&preOpenInterest, &preOpenInterest);
        Tranverse8Byte(&preClose, &preClose);
        Tranverse8Byte(&preSettlePx, &preSettlePx);
        Tranverse8Byte(&open, &open);
        Tranverse8Byte(&high, &high);
        Tranverse8Byte(&low, &low);
        Tranverse8Byte(&last, &last);
        Tranverse8Byte(&volume, &volume);
        Tranverse8Byte(&turnover, &turnover);
        Tranverse8Byte(&openInterest, &openInterest);
        Tranverse8Byte(&close, &close);
        Tranverse8Byte(&settlePrice, &settlePrice);
        Tranverse8Byte(&highLimited, &highLimited);
        Tranverse8Byte(&lowLimited, &lowLimited);
        Tranverse4Byte(&preDelta, &preDelta);
        Tranverse4Byte(&currDelta, &currDelta);
        for (int i = 0; i < 5; i++) {
            Tranverse8Byte(&askPx[i], &askPx[i]);
            Tranverse8Byte(&askVol[i], &askVol[i]);
            Tranverse8Byte(&bidPx[i], &bidPx[i]);
            Tranverse8Byte(&bidVol[i], &bidVol[i]);
        }
        Tranverse8Byte(&auctionPx, &auctionPx);
        Tranverse8Byte(&auctionQty, &auctionQty);
    }
};

///上交所逐笔委托(202105新增)
// 成交类型
using CSHOrderTypeType = char;
///新增
#define SH_ORDER_TYPE_ADD 'A'
///撤单
#define SH_ORDER_TYPE_DEL 'D'
struct REMD_SH_OrderDetail {
    ///代码
    REMD_Code code;
    ///业务时间戳YYYYMMDDHHMMSSsss
    int64_t     timeStamp;
    ///委托号
    int32_t orderIndex;
    ///通道号
    int32_t channelNo;
    ///委托价格
    int64_t orderPrice;
    ///剩余委托量
    int64_t balance;
    ///原始委托号
    int64_t orderNo;
    ///订单类型(enum: SHOrderType)
    CSHOrderTypeType orderType;
    ///内外盘类型(enum: BSFlag)
    REMD_BSFlag BSFlag;
    ///业务号
    int64_t bizIndex;
    ///数据状态，直接发送上交所数据
    char dataStatus;
    void EndianSwap() {
        Tranverse8Byte(&timeStamp, &timeStamp);
        Tranverse4Byte(&orderIndex, &orderIndex);
        Tranverse4Byte(&channelNo, &channelNo);
        Tranverse8Byte(&orderPrice, &orderPrice);
        Tranverse8Byte(&balance, &balance);
        Tranverse8Byte(&orderNo, &orderNo);
        Tranverse8Byte(&bizIndex, &bizIndex);
    }
};
#pragma pack()

static const std::size_t REMD_PKT_SIZE = sizeof(RemdPacket);
static const std::size_t STRUCT_FIELD_SIZE = sizeof(StructFieldHeader);