﻿#ifndef __API_TEST_STRUCT_DATA_TEST_H__
#define __API_TEST_STRUCT_DATA_TEST_H__

#include "test_base.h"
#include "sys/stat.h"
#include <sys/types.h>
#include <dirent.h>
#include "net.h"
#include "remd_struct.h"

REMD_IndexSnapShot* pIdxData(nullptr);
REMD_SecuritySnapShot* pSnapData(nullptr);
REMD_OptionSnapShot* pFutData(nullptr);
REMD_TradeDetail* pTrdData(nullptr);
REMD_OrderDetail* pOrdData(nullptr);
REMD_OrderQueue* pQueData(nullptr);
const static std::size_t SIZE_REMD_INX_SNP = sizeof(REMD_IndexSnapShot);
const static std::size_t SIZE_REMD_SEC_SNP = sizeof(REMD_SecuritySnapShot);
const static std::size_t SIZE_REMD_ORD_QUE = sizeof(REMD_OrderQueue);
const static std::size_t SIZE_REMD_ORD_DTL = sizeof(REMD_OrderDetail);
const static std::size_t SIZE_REMD_TRD_DTL = sizeof(REMD_TradeDetail);
const static std::size_t SIZE_REMD_FUT_SNP = sizeof(REMD_OptionSnapShot);
const static std::size_t SIZE_REMD_SH_ORD_DTL = sizeof(REMD_SH_OrderDetail);
struct timespec localTs;

///服务器时间戳（HHMMSSmmm）转换为绝对ms
static inline uint64_t ConvertServerTm2MSec(int tm) {
    uint64_t hsec = tm / 10000000 * 3600;
    uint64_t msec = tm % 10000000 / 100000 * 60;
    uint64_t sec = tm % 100000 / 1000;
    return (hsec + msec + sec) * 1000 + tm % 1000;
}

class StructDataTest {
    using this_type = StructDataTest;
public:

    static inline void Data2Index(FILE* fp, unsigned char* pData) {
        REMD_IndexSnapShot data;
        memcpy(&data, pData, SIZE_REMD_INX_SNP);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static inline void Data2Snap(FILE* fp, unsigned char* pData) {
        REMD_SecuritySnapShot data;
        memcpy(&data, pData, SIZE_REMD_SEC_SNP);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static inline std::size_t Data2OrderQueue(FILE* fp, unsigned char* pData) {
        REMD_OrderQueue data;
        memcpy(&data, pData, SIZE_REMD_ORD_QUE);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static inline void Data2OrderDetail(FILE* fp, unsigned char* pData) {
        REMD_OrderDetail data;
        memcpy(&data, pData, SIZE_REMD_ORD_DTL);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static inline void Data2TradeDetail(FILE* fp, unsigned char* pData) {
        REMD_TradeDetail data;
        memcpy(&data, pData, SIZE_REMD_TRD_DTL);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static inline void Data2Option(FILE* fp, unsigned char* pData) {
        REMD_OptionSnapShot data;
        memcpy(&data, pData, SIZE_REMD_FUT_SNP);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static inline void Data2SHOrderDetail(FILE* fp, unsigned char* pData) {
        REMD_SH_OrderDetail data;
        memcpy(&data, pData, SIZE_REMD_SH_ORD_DTL);
        data.EndianSwap();
        ToFile(fp, &data, *PrintMdHeader());
    }

    static void OnProcRcvData(unsigned char* buff, std::size_t rcvcnt) {
        ///得到本地时间
        clock_gettime(CLOCK_REALTIME, &localTs);
        const char* pStrLocTm = make_time(localTs.tv_sec).data();

        ///行情处理条数
        static uint64_t mdCount(0);

        ///接收到的数据remd包结构
        RemdPacket pkt;
        std::size_t procSize = REMD_PKT_SIZE;
        StructFieldHeader fieldHdr;
        //const std::size_t FIELD_SIZE = sizeof(StructFieldHeader);
        FILE* fp = *DumpFile();
        while (true) {
            if (rcvcnt - (procSize + STRUCT_FIELD_SIZE) <= 0) {
                break;
            }
            memcpy(&fieldHdr, buff + procSize, STRUCT_FIELD_SIZE);
            procSize += STRUCT_FIELD_SIZE;
            fieldHdr.EndianSwap();

            ///丢包判断
            static uint64_t nSeq = fieldHdr.Sequence;
            if (*DetectLost()) {
                nSeq++;
                if (nSeq < fieldHdr.Sequence) {
                    printf("Err[lost]%s seq:%ld,Lost:%ld\n", pStrLocTm, fieldHdr.Sequence, fieldHdr.Sequence - nSeq);
                    nSeq = fieldHdr.Sequence;
                }
            }

            ///打本地时间戳
            if (fp) {
                fprintf(fp, "%ld.%ld,%ld,", localTs.tv_sec, localTs.tv_nsec, fieldHdr.Sequence);
            }

            //if (fieldHdr.FID != FID_TYPE::SH_TRDDTL)break;

            switch (fieldHdr.FID) {
            case FID_TYPE::SNAP:
                Data2Snap(fp, buff + procSize);
                procSize += SIZE_REMD_SEC_SNP;
                break;
            case FID_TYPE::INDEX:
                Data2Index(fp, buff + procSize);
                procSize += SIZE_REMD_INX_SNP;
                break;
            case FID_TYPE::ORDQUE:
                procSize += Data2OrderQueue(fp, buff + procSize);
                procSize += SIZE_REMD_ORD_QUE;
                break;
            case FID_TYPE::ORDDTL:
                Data2OrderDetail(fp, buff + procSize);
                procSize += SIZE_REMD_ORD_DTL;
                break;
            case FID_TYPE::TRDDTL:
                Data2TradeDetail(fp, buff + procSize);
                procSize += SIZE_REMD_TRD_DTL;
                break;
            case FID_TYPE::OPTIONS:
                Data2Option(fp, buff + procSize);
                procSize += SIZE_REMD_FUT_SNP;
                break;
            case FID_TYPE::SH_TRDDTL:
                Data2SHOrderDetail(fp, buff + procSize);
                procSize += SIZE_REMD_SH_ORD_DTL;
                break;
            }

            if (procSize >= rcvcnt) {
                break;
            }
        }

        mdCount++;
        if (0 == mdCount % 10000) {
            printf("%s md +%d\n", pStrLocTm, mdCount);
        }
    }

    static void RunRcvTask(TMulticastConfig& rcvConf, const std::size_t MAX_RCV_BUF_SIZE) {
        ///初始化组播接收
        unsigned char buf[MAX_RCV_BUF_SIZE];
        int32_t rcvcnt(0);
        struct sockaddr_in fromAddr;
        int fromLen = sizeof(fromAddr);
        if (true == InitMulticastReceive(rcvConf, true)) {
            ///开始接收数据
            while (true) {
                memset(buf, 0, MAX_RCV_BUF_SIZE);
                rcvcnt = ::recvfrom(rcvConf.socket, buf, MAX_RCV_BUF_SIZE, 0, (struct sockaddr*) & fromAddr, (socklen_t*)&fromLen);
                if (rcvcnt > 0) {
                    OnProcRcvData(buf, rcvcnt);
                    //printf("rcv pkt(%d)\n", rcvcnt);
                }
            }
        }
    }

    static void RunRawRcvTask(TMulticastConfig& rcvConf, const std::size_t MAX_RCV_BUF_SIZE) {
        uint32_t dstIpInt = Ipv4ToInteger(rcvConf.mcIp);
        int dstPort = rcvConf.mcPort;

        TEthernetHdr ethHdr;
        const uint32_t ETH_HDR_SIZE = sizeof(TEthernetHdr);
        TUDPHeader udphdr;
        const uint32_t UDP_HDR_SIZE = sizeof(TUDPHeader);

        ///初始化组播接收
        unsigned char buf[MAX_RCV_BUF_SIZE];
        int32_t rcvcnt(0);
        struct sockaddr_in fromAddr;
        int fromLen = sizeof(fromAddr);
        if (!InitMiscModeReceive(rcvConf)) {
            printf("Failed to InitMiscModeReceive() ");
            return;
        }

        ///开始接收数据
        while (true) {
            memset(buf, 0, MAX_RCV_BUF_SIZE);
            rcvcnt = ::recvfrom(rcvConf.socket, buf, MAX_RCV_BUF_SIZE, 0, (struct sockaddr*)&fromAddr, (socklen_t*)&fromLen);
            
            if (rcvcnt > ETH_HDR_SIZE) {
                memcpy(&ethHdr, buf, ETH_HDR_SIZE);

                if (IPPROTO_UDP == ethHdr.iphdr.Protocol && dstIpInt == ethHdr.iphdr.DstIP) {
                    memcpy(&udphdr, buf + ETH_HDR_SIZE, UDP_HDR_SIZE);
                    Tranverse2Byte(&udphdr.DstPort, &udphdr.DstPort);

                    if (dstPort == udphdr.DstPort) {
                        OnProcRcvData(buf + ETH_HDR_SIZE + UDP_HDR_SIZE, rcvcnt);
                    }
                }
            }
        }
    }

    static int Do(const remd::IniParser::Section* section) {
        //Load configs 
        TMulticastConfig rcvConf;
        rcvConf.mcIp = get_value_and_check(section, "multicast_ip");
        std::string value = get_value_and_check(section, "multicast_port");
        rcvConf.mcPort = strtoul(value.c_str(), nullptr, 10);
        rcvConf.localIp = get_value_and_check(section, "multicast_interface");

        ///接收buff大小,根据实际业务需求设置，如跨交换机则小于默认MTU(1500)
        value = get_value_and_check(section, "max_recv_buff_size");
        const int MAX_RCV_BUF_SIZE = strtol(value.c_str(), nullptr, 10);

        value = get_value_and_check(section, "print_header");
        bool* phptr = PrintMdHeader();
        *phptr = stoi(value);

        bool* foptr = EnableFileOut();
        value = get_value_and_check(section, "file_out");
        *foptr = stoi(value);

        //丢包检测
        value = get_value_and_check(section, "lost_detect");
        *DetectLost() = stoi(value);

        //延时判断
        value = get_value_and_check(section, "delay_threshold");
        *DelayThresholdMs() = stoi(value);
        if (*DelayThresholdMs() <= 0) {
            *DelayThresholdMs() = 2000;
        }

        if (*EnableFileOut()) {
            std::string sOutFolder("./data/");
            std::string dump(sOutFolder);
            time_t now = time(0);
            tm* ltm = localtime(&now);
            char iyear[50], imonth[50], iday[50], ihour[50], imin[50];
            sprintf(iyear, "%d", 1900 + ltm->tm_year);
            sprintf(imonth, "%02d", 1 + ltm->tm_mon);
            sprintf(iday, "%02d", ltm->tm_mday);
            sprintf(ihour, "%02d", ltm->tm_hour);
            sprintf(imin, "%02d", ltm->tm_min);

            //20180728
            dump += iyear;
            dump += imonth;
            dump += iday;
            dump += ihour;
            dump += imin;
            dump += ".txt";
            if (!dump.empty()) {
                if (nullptr == opendir(sOutFolder.c_str())) {
                    mkdir(sOutFolder.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
                }

                auto fptr = DumpFile();
                *fptr = fopen(dump.c_str(), "w");
                if (*fptr) {
                    printf("create file: %s.\n", dump.c_str());
                }
                else {
                    printf("create file: %s failed!!\n", dump.c_str());
                }
            }
        }
        else {//print to screen
            *DumpFile() = stdout;
            printf("en_print_file = true.\n");
        }

        ///开始接收数据
        //RunRcvTask(rcvConf, MAX_RCV_BUF_SIZE);
        RunRawRcvTask(rcvConf, MAX_RCV_BUF_SIZE);
        return 0;
    }

    static bool* EnableFileOut() {
        static bool fileout = false;
        return &fileout;
    }

    static FILE** DumpFile() {
        static FILE* fptr = nullptr;
        return &fptr;
    }

    ///异常检测
    static bool* DetectLost() {
        static bool lost = false;
        return &lost;
    }

    ///异常检测延时阈值
    static int32_t* DelayThresholdMs() {
        static int32_t threshold = 0;
        return &threshold;
    }

    ///是否打印每个字段的标题
    static bool* PrintMdHeader() {
        static bool printHeader = false;
        return &printHeader;
    }
};

#endif // __API_TEST_STRUCT_DATA_TEST_H__
