﻿// @Author:   citicsf
// @Date:     2019/08/24
// @Brief:

#ifndef __TEST_BASE_H__
#define __TEST_BASE_H__

#include <pthread.h>
#include "output_struct.h"
#include "assert.h"
#include "ini_parser.h"

std::string get_value_and_check(const remd::IniParser::Section* section,
    const char* key) {
    std::string value = section->GetValue(key);
    if (value.empty()) {
        printf("value empty: %s:%s\n", section->Name(), key);
        throw std::string("bad config");
    }
    return value;
}

inline std::string make_datetime(int32_t sec)
{
    char buf[80];
    tm d_tm;
    time_t tvsec = sec;
#if defined( _WIN32 )
    localtime_s(&d_tm, &tvsec);
#else
    localtime_r(&tvsec, &d_tm);
#endif
    std::size_t chars = ::strftime(buf, 80, "%Y%m%d %H:%M:%S", &d_tm);
    return std::string(buf, chars);
}

inline std::string make_time(int32_t sec)
{
    char buf[10];
    tm d_tm;
    time_t tvsec = sec;
#if defined( _WIN32 )
    localtime_s(&d_tm, &tvsec);
#else
    localtime_r(&tvsec, &d_tm);
#endif
    std::size_t chars = ::strftime(buf, 10, "%H%M%S", &d_tm);
    return std::string(buf, chars);
}

inline std::vector<std::string>
StringSplit(const std::string& src, char split) {
    std::vector<std::string> tar;
    auto idx_s = src.begin(), idx_e = src.begin(), str_end = src.end();
    for (; idx_e < str_end; ++idx_e) {
        if (*idx_e == split && idx_e != idx_s) {
            tar.emplace_back(idx_s, idx_e);
            idx_s = idx_e + 1;
        }
    }
    if (idx_e != idx_s) {
        tar.emplace_back(idx_s, idx_e);
    }
    return tar;
}

void bind_2_cpu(int cpu_id) {
    cpu_set_t cpu_set;
    ///初始化，设为空
    CPU_ZERO(&cpu_set);
    ///将某个cpu加入cpu集中
    CPU_SET(cpu_id, &cpu_set);
    pthread_setaffinity_np(pthread_self(), sizeof(cpu_set), &cpu_set);
}



#endif // __TEST_BASE_H__
