#include "userfunc.hpp"
#include "FixSseApiStruct.hpp"

namespace xele_md
{
    uint64_t htonl_64(uint64_t val)
    {
#ifndef NOBYTEORD
         #if __BYTE_ORDER == __LITTLE_ENDIAN
         union { uint32_t lv[2]; uint64_t llv; } u;
         u.lv[0] = htonl(val >> 32);
         u.lv[1] = htonl(val & 0xFFFFFFFFULL);
         return u.llv;
         #endif
#endif
         return val;
    }

    double ntoh_double(double val)
    {
         double ret = val;
#ifndef NOBYTEORD
         #if __BYTE_ORDER == __LITTLE_ENDIAN
         uint64_t tmp = htonl_64(*((uint64_t *)&val));
         ret = *((double*)&tmp);
         #endif
#endif
         return ret;
    }

    int code_convert(const char* from_charset, const char* to_charset, char* inbuf, size_t inlen, char* outbuf, size_t outlen)
    {
         iconv_t cd;
         char **pin = &inbuf;
         char **pout = &outbuf;

         cd = iconv_open(to_charset,from_charset);
         if (cd == 0)
         {
              return -1;
         }
         memset(outbuf,0,outlen);
         if (iconv(cd, pin, &inlen, pout, &outlen) == (size_t)-1)
         {
              return -1;
         }

         iconv_close(cd);
         return 0;
    }

    int gbk_to_utf8(char* inbuf, size_t inlen, char* outbuf, size_t outlen)
    {
         return code_convert("gbk", "utf-8", inbuf, inlen, outbuf, outlen);
    }

    void Print_Sse_MarketData_Stock(FILE* logfd, struct Sse_MarketData_Stock obj)
    {

         printf("\n");
         printf("MdStreamID:%s\n",obj.MdStreamID);
         fprintf(logfd,"MdStreamID:%s\n",obj.MdStreamID);

         int k = 0;
         printf("seq_num:%lu\n", htonl_64(obj.seq_num));
         fprintf(logfd,"seq_num:%lu\n", htonl_64(obj.seq_num));

         printf("Timestamp:");
         fprintf(logfd,"Timestamp:");
         for (int i = 0;i < 8;++i)
         {
              if (obj.Timestamp[i] != 0)
              {
                   printf("%c",obj.Timestamp[i]);
                   fprintf(logfd,"%c",obj.Timestamp[i]);
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("SecutrityID:");
         fprintf(logfd,"SecutrityID:");
         for (int i = 0;i < 8;++i)
         {
              if (obj.SecutrityID[i] != 0)
              {
                   printf("%c",obj.SecutrityID[i]);
                   fprintf(logfd,"%c",obj.SecutrityID[i]);
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("Symbol:");
         fprintf(logfd,"Symbol:");
         k = 0;
         while (k < 8)
         {
              if (obj.Symbol[k] >= 0x00 && obj.Symbol[k] <= 0x7F)
              {
                   printf("%c",obj.Symbol[k]);
                   fprintf(logfd,"%c",obj.Symbol[k]);
                   ++k;
              }
              else
              {
                   char str_gbk[3] = "";
                   char buff[5]= "";
                   str_gbk[0] = obj.Symbol[k];
                   ++k;
                   str_gbk[1] = obj.Symbol[k];
                   gbk_to_utf8(str_gbk,strlen(str_gbk),buff,sizeof(buff));
                   printf("%s",buff);
                   fprintf(logfd,"%s",buff);
                   ++k;
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("TradeVolume:%lu\n",htonl_64(obj.TradeVolume));
         fprintf(logfd,"TradeVolume:%lu\n",htonl_64(obj.TradeVolume));
         printf("TotalValueTraded:%0.2lf\n",ntoh_double(obj.TotalValueTraded));
         fprintf(logfd,"TotalValueTraded:%0.2lf\n",ntoh_double(obj.TotalValueTraded));
         printf("PreClosePrice:%0.2lf\n",ntoh_double(obj.PreClosePrice));
         fprintf(logfd,"PreClosePrice:%0.2lf\n",ntoh_double(obj.PreClosePrice));
         printf("PrevSetPrice:%0.2lf\n",ntoh_double(obj.PrevSetPrice));
         fprintf(logfd,"PrevSetPrice:%0.2lf\n",ntoh_double(obj.PrevSetPrice));
         printf("TotalLongPosition:%lu\n",htonl_64(obj.TotalLongPosition));
         fprintf(logfd,"TotalLongPosition:%lu\n",htonl_64(obj.TotalLongPosition));
         printf("IndexValue:%0.2lf\n",ntoh_double(obj.IndexValue));
         fprintf(logfd,"IndexValue:%0.2lf\n",ntoh_double(obj.IndexValue));
         printf("OpenPrice:%0.2lf\n",ntoh_double(obj.OpenPrice));
         fprintf(logfd,"OpenPrice:%0.2lf\n",ntoh_double(obj.OpenPrice));
         printf("ClosePrice:%0.2lf\n",ntoh_double(obj.ClosePrice));
         fprintf(logfd,"ClosePrice:%0.2lf\n",ntoh_double(obj.ClosePrice));
         printf("HighPrice:%0.2lf\n",ntoh_double(obj.HighPrice));
         fprintf(logfd,"HighPrice:%0.2lf\n",ntoh_double(obj.HighPrice));
         printf("LowPrice:%0.2lf\n",ntoh_double(obj.LowPrice));
         fprintf(logfd,"LowPrice:%0.2lf\n",ntoh_double(obj.LowPrice));
         printf("TradingPhaseCode:%s\n",obj.TradingPhaseCode);
         fprintf(logfd,"TradingPhaseCode:%s\n",obj.TradingPhaseCode);
         printf("--------------------------------\n");
         fprintf(logfd,"%s","--------------------------------\n");
         printf("Input 'q' to disconnect API:");
         fflush(stdout);
    }

    void Print_Sse_MarketData_Other(FILE* logfd,struct Sse_MarketData_Other obj)
    {
         printf("\n");
         printf("MdStreamID:%s\n",obj.MdStreamID);
         fprintf(logfd,"MdStreamID:%s\n",obj.MdStreamID);

         int k = 0;
         printf("seq_num:%lu\n", htonl_64(obj.seq_num));
         fprintf(logfd,"seq_num:%lu\n", htonl_64(obj.seq_num));

         printf("Timestamp:");
         fprintf(logfd,"Timestamp:");
         for (int i = 0;i < 8;++i)
         {
              if (obj.Timestamp[i] != 0)
              {
                   printf("%c",obj.Timestamp[i]);
                   fprintf(logfd,"%c",obj.Timestamp[i]);
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("SecutrityID:");
         fprintf(logfd,"SecutrityID:");
         for (int i = 0;i < 8;++i)
         {
              if (obj.SecutrityID[i] != 0)
              {
                   printf("%c",obj.SecutrityID[i]);
                   fprintf(logfd,"%c",obj.SecutrityID[i]);
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("Symbol:");
         fprintf(logfd,"Symbol:");
         k = 0;
         while (k < 8)
         {
              if (obj.Symbol[k] >= 0x00 && obj.Symbol[k] <= 0x7F)
              {
                   printf("%c",obj.Symbol[k]);
                   fprintf(logfd,"%c",obj.Symbol[k]);
                   ++k;
              }
              else
              {
                   char str_gbk[3] = "";
                   char buff[5]= "";
                   str_gbk[0] = obj.Symbol[k];
                   ++k;
                   str_gbk[1] = obj.Symbol[k];
                   gbk_to_utf8(str_gbk,strlen(str_gbk),buff,sizeof(buff));
                   printf("%s",buff);
                   fprintf(logfd,"%s",buff);
                   ++k;
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("TradeVolume:%lu\n",htonl_64(obj.TradeVolume));
         fprintf(logfd,"TradeVolume:%lu\n",htonl_64(obj.TradeVolume));
         printf("TotalValueTraded:%0.2lf\n",ntoh_double(obj.TotalValueTraded));
         fprintf(logfd,"TotalValueTraded:%0.2lf\n",ntoh_double(obj.TotalValueTraded));
         printf("PreClosePrice:%0.2lf\n",ntoh_double(obj.PreClosePrice));
         fprintf(logfd,"PreClosePrice:%0.2lf\n",ntoh_double(obj.PreClosePrice));
         printf("PrevSetPrice:%0.2lf\n",ntoh_double(obj.PrevSetPrice));
         fprintf(logfd,"PrevSetPrice:%0.2lf\n",ntoh_double(obj.PrevSetPrice));
         printf("TotalLongPosition:%lu\n",htonl_64(obj.TotalLongPosition));
         fprintf(logfd,"TotalLongPosition:%lu\n",htonl_64(obj.TotalLongPosition));
         printf("BidPrice1:%0.2lf\n",ntoh_double(obj.BidPrice1));
         fprintf(logfd,"BidPrice1:%0.2lf\n",ntoh_double(obj.BidPrice1));
         printf("BidVolume1:%lu\n",htonl_64(obj.BidVolume1));
         fprintf(logfd,"BidVolume1:%lu\n",htonl_64(obj.BidVolume1));
         printf("BidPrice2:%0.2lf\n",ntoh_double(obj.BidPrice2));
         fprintf(logfd,"BidPrice2:%0.2lf\n",ntoh_double(obj.BidPrice2));
         printf("BidVolume2:%lu\n",htonl_64(obj.BidVolume2));
         fprintf(logfd,"BidVolume2:%lu\n",htonl_64(obj.BidVolume2));
         printf("BidPrice3:%0.2lf\n",ntoh_double(obj.BidPrice3));
         fprintf(logfd,"BidPrice3:%0.2lf\n",ntoh_double(obj.BidPrice3));
         printf("BidVolume3:%lu\n",htonl_64(obj.BidVolume3));
         fprintf(logfd,"BidVolume3:%lu\n",htonl_64(obj.BidVolume3));
         printf("BidPrice4:%0.2lf\n",ntoh_double(obj.BidPrice4));
         fprintf(logfd,"BidPrice4:%0.2lf\n",ntoh_double(obj.BidPrice4));
         printf("BidVolume4:%lu\n",htonl_64(obj.BidVolume4));
         fprintf(logfd,"BidVolume4:%lu\n",htonl_64(obj.BidVolume4));
         printf("BidPrice5:%0.2lf\n",ntoh_double(obj.BidPrice5));
         fprintf(logfd,"BidPrice5:%0.2lf\n",ntoh_double(obj.BidPrice5));
         printf("BidVolume5:%lu\n",htonl_64(obj.BidVolume5));
         fprintf(logfd,"BidVolume5:%lu\n",htonl_64(obj.BidVolume5));
         printf("AskPrice1:%0.2lf\n",ntoh_double(obj.AskPrice1));
         fprintf(logfd,"AskPrice1:%0.2lf\n",ntoh_double(obj.AskPrice1));
         printf("AskVolume1:%lu\n",htonl_64(obj.AskVolume1));
         fprintf(logfd,"AskVolume1:%lu\n",htonl_64(obj.AskVolume1));
         printf("AskPrice2:%0.2lf\n",ntoh_double(obj.AskPrice2));
         fprintf(logfd,"AskPrice2:%0.2lf\n",ntoh_double(obj.AskPrice2));
         printf("AskVolume2:%lu\n",htonl_64(obj.AskVolume2));
         fprintf(logfd,"AskVolume2:%lu\n",htonl_64(obj.AskVolume2));
         printf("AskPrice3:%0.2lf\n",ntoh_double(obj.AskPrice3));
         fprintf(logfd,"AskPrice3:%0.2lf\n",ntoh_double(obj.AskPrice3));
         printf("AskVolume3:%lu\n",htonl_64(obj.AskVolume3));
         fprintf(logfd,"AskVolume3:%lu\n",htonl_64(obj.AskVolume3));
         printf("AskPrice4:%0.2lf\n",ntoh_double(obj.AskPrice4));
         fprintf(logfd,"AskPrice4:%0.2lf\n",ntoh_double(obj.AskPrice4));
         printf("AskVolume4:%lu\n",htonl_64(obj.AskVolume4));
         fprintf(logfd,"AskVolume4:%lu\n",htonl_64(obj.AskVolume4));
         printf("AskPrice5:%0.2lf\n",ntoh_double(obj.AskPrice5));
         fprintf(logfd,"AskPrice5:%0.2lf\n",ntoh_double(obj.AskPrice5));
         printf("AskVolume5:%lu\n",htonl_64(obj.AskVolume5));
         fprintf(logfd,"AskVolume5:%lu\n",htonl_64(obj.AskVolume5));
         printf("TradePrice:%0.2lf\n",ntoh_double(obj.TradePrice));
         fprintf(logfd,"TradePrice:%0.2lf\n",ntoh_double(obj.TradePrice));
         printf("OpenPrice:%0.2lf\n",ntoh_double(obj.OpenPrice));
         fprintf(logfd,"OpenPrice:%0.2lf\n",ntoh_double(obj.OpenPrice));
         printf("ClosePrice:%0.2lf\n",ntoh_double(obj.ClosePrice));
         fprintf(logfd,"ClosePrice:%0.2lf\n",ntoh_double(obj.ClosePrice));
         printf("HighPrice:%0.2lf\n",ntoh_double(obj.HighPrice));
         fprintf(logfd,"HighPrice:%0.2lf\n",ntoh_double(obj.HighPrice));
         printf("LowPrice:%0.2lf\n",ntoh_double(obj.LowPrice));
         fprintf(logfd,"LowPrice:%0.2lf\n",ntoh_double(obj.LowPrice));

         if (strcmp((char*)obj.MdStreamID,"MD004") == 0)
         {
              printf("IOPV:%0.2lf\n",ntoh_double(obj.IOPV));
              fprintf(logfd,"IOPV:%0.2lf\n",ntoh_double(obj.IOPV));

              printf("PreCloseIOPV:%0.2lf\n",ntoh_double(obj.PreCloseIOPV));
              fprintf(logfd,"PreCloseIOPV:%0.2lf\n",ntoh_double(obj.PreCloseIOPV));
         }

         printf("TradingPhaseCode:%s\n",obj.TradingPhaseCode);
         fprintf(logfd,"TradingPhaseCode:%s\n",obj.TradingPhaseCode);
         printf("--------------------------------\n");
         fprintf(logfd,"%s","--------------------------------\n");
         printf("Input 'q' to disconnect API:");
         fflush(stdout);
    }

    void Print_Sse_MarketData_Option(FILE* logfd,struct Sse_MarketData_Option obj)
    {
         printf("\n");
         printf("MdStreamID:%s\n",obj.MdStreamID);
         fprintf(logfd,"MdStreamID:%s\n",obj.MdStreamID);

         printf("seq_num:%lu\n", htonl_64(obj.seq_num));
         fprintf(logfd, "seq_num:%lu\n", htonl_64(obj.seq_num));

         printf("Timestamp:");
         fprintf(logfd,"Timestamp:");
         for (int i = 0;i < 8; ++i)
         {
              if (obj.Timestamp[i] != 0)
              {
                   printf("%c",obj.Timestamp[i]);
                   fprintf(logfd,"%c",obj.Timestamp[i]);
              }
         }
         printf("\n");
         fprintf(logfd,"\n");

         printf("SecutrityID:");
         fprintf(logfd,"SecutrityID:");
         for (int i = 0;i < 8;++i)
         {
              if (obj.SecutrityID[i] != 0)
              {
                   printf("%c",obj.SecutrityID[i]);
                   fprintf(logfd,"%c",obj.SecutrityID[i]);
              }
         }
         printf("\n");
         fprintf(logfd,"\n");
         /*
         printf("Symbol:");
         k = 0;
         while (k < 8)
         {
             if (obj.Symbol[k] >= 0x00 && obj.Symbol[k] <= 0x7F)
             {
                 printf("%c",obj.Symbol[k]);
                 ++k;
             }
             else
             {
                 char str_gbk[3] = "";
                 char buff[5]= "";
                 str_gbk[0] = obj.Symbol[k];
                 ++k;
                 str_gbk[1] = obj.Symbol[k];
                 gbk_to_utf8(str_gbk,strlen(str_gbk),buff,sizeof(buff));
                 printf("%s",buff);
                 ++k;
             }
         }
         printf("\n");
         */
         printf("TradeVolume:%lu\n",htonl_64(obj.TradeVolume));
         fprintf(logfd,"TradeVolume:%lu\n",htonl_64(obj.TradeVolume));
         printf("TotalValueTraded:%0.4lf\n",ntoh_double(obj.TotalValueTraded));
         fprintf(logfd,"TotalValueTraded:%0.4lf\n",ntoh_double(obj.TotalValueTraded));
         printf("PreClosePrice:%0.4lf\n",ntoh_double(obj.PreClosePrice));
         fprintf(logfd,"PreClosePrice:%0.4lf\n",ntoh_double(obj.PreClosePrice));
         printf("PrevSetPrice:%0.4lf\n",ntoh_double(obj.PrevSetPrice));
         fprintf(logfd,"PrevSetPrice:%0.4lf\n",ntoh_double(obj.PrevSetPrice));
         printf("TotalLongPosition:%lu\n",htonl_64(obj.TotalLongPosition));
         fprintf(logfd,"TotalLongPosition:%lu\n",htonl_64(obj.TotalLongPosition));
         printf("BidPrice1:%0.4lf\n",ntoh_double(obj.BidPrice1));
         fprintf(logfd,"BidPrice1:%0.4lf\n",ntoh_double(obj.BidPrice1));
         printf("BidVolume1:%lu\n",htonl_64(obj.BidVolume1));
         fprintf(logfd,"BidVolume1:%lu\n",htonl_64(obj.BidVolume1));
         printf("BidPrice2:%0.4lf\n",ntoh_double(obj.BidPrice2));
         fprintf(logfd,"BidPrice2:%0.4lf\n",ntoh_double(obj.BidPrice2));
         printf("BidVolume2:%lu\n",htonl_64(obj.BidVolume2));
         fprintf(logfd,"BidVolume2:%lu\n",htonl_64(obj.BidVolume2));
         printf("BidPrice3:%0.4lf\n",ntoh_double(obj.BidPrice3));
         fprintf(logfd,"BidPrice3:%0.4lf\n",ntoh_double(obj.BidPrice3));
         printf("BidVolume3:%lu\n",htonl_64(obj.BidVolume3));
         fprintf(logfd,"BidVolume3:%lu\n",htonl_64(obj.BidVolume3));
         printf("BidPrice4:%0.4lf\n",ntoh_double(obj.BidPrice4));
         fprintf(logfd,"BidPrice4:%0.4lf\n",ntoh_double(obj.BidPrice4));
         printf("BidVolume4:%lu\n",htonl_64(obj.BidVolume4));
         fprintf(logfd,"BidVolume4:%lu\n",htonl_64(obj.BidVolume4));
         printf("BidPrice5:%0.4lf\n",ntoh_double(obj.BidPrice5));
         fprintf(logfd,"BidPrice5:%0.4lf\n",ntoh_double(obj.BidPrice5));
         printf("BidVolume5:%lu\n",htonl_64(obj.BidVolume5));
         fprintf(logfd,"BidVolume5:%lu\n",htonl_64(obj.BidVolume5));
         printf("AskPrice1:%0.4lf\n",ntoh_double(obj.AskPrice1));
         fprintf(logfd,"AskPrice1:%0.4lf\n",ntoh_double(obj.AskPrice1));
         printf("AskVolume1:%lu\n",htonl_64(obj.AskVolume1));
         fprintf(logfd,"AskVolume1:%lu\n",htonl_64(obj.AskVolume1));
         printf("AskPrice2:%0.4lf\n",ntoh_double(obj.AskPrice2));
         fprintf(logfd,"AskPrice2:%0.4lf\n",ntoh_double(obj.AskPrice2));
         printf("AskVolume2:%lu\n",htonl_64(obj.AskVolume2));
         fprintf(logfd,"AskVolume2:%lu\n",htonl_64(obj.AskVolume2));
         printf("AskPrice3:%0.4lf\n",ntoh_double(obj.AskPrice3));
         fprintf(logfd,"AskPrice3:%0.4lf\n",ntoh_double(obj.AskPrice3));
         printf("AskVolume3:%lu\n",htonl_64(obj.AskVolume3));
         fprintf(logfd,"AskVolume3:%lu\n",htonl_64(obj.AskVolume3));
         printf("AskPrice4:%0.4lf\n",ntoh_double(obj.AskPrice4));
         fprintf(logfd,"AskPrice4:%0.4lf\n",ntoh_double(obj.AskPrice4));
         printf("AskVolume4:%lu\n",htonl_64(obj.AskVolume4));
         fprintf(logfd,"AskVolume4:%lu\n",htonl_64(obj.AskVolume4));
         printf("AskPrice5:%0.4lf\n",ntoh_double(obj.AskPrice5));
         fprintf(logfd,"AskPrice5:%0.4lf\n",ntoh_double(obj.AskPrice5));
         printf("AskVolume5:%lu\n",htonl_64(obj.AskVolume5));
         fprintf(logfd,"AskVolume5:%lu\n",htonl_64(obj.AskVolume5));
         printf("TradePrice:%0.4lf\n",ntoh_double(obj.TradePrice));
         fprintf(logfd,"TradePrice:%0.4lf\n",ntoh_double(obj.TradePrice));
         printf("OpenPrice:%0.4lf\n",ntoh_double(obj.OpenPrice));
         fprintf(logfd,"OpenPrice:%0.4lf\n",ntoh_double(obj.OpenPrice));
         printf("ClosePrice:%0.4lf\n",ntoh_double(obj.ClosePrice));
         fprintf(logfd,"ClosePrice:%0.4lf\n",ntoh_double(obj.ClosePrice));
         printf("SettPrice:%0.4lf\n",ntoh_double(obj.SettlPrice));
         fprintf(logfd,"SettPrice:%0.4lf\n",ntoh_double(obj.SettlPrice));
         printf("HighPrice:%0.4lf\n",ntoh_double(obj.HighPrice));
         fprintf(logfd,"HighPrice:%0.4lf\n",ntoh_double(obj.HighPrice));
         printf("LowPrice:%0.4lf\n",ntoh_double(obj.LowPrice));
         fprintf(logfd,"LowPrice:%0.4lf\n",ntoh_double(obj.LowPrice));
         printf("AuctionPrice:%0.4lf\n",ntoh_double(obj.AuctionPrice));
         fprintf(logfd,"AuctionPrice:%0.4lf\n",ntoh_double(obj.AuctionPrice));
         printf("AuctionQty:%lu\n",htonl_64(obj.AuctionQty));
         fprintf(logfd,"AuctionQty:%lu\n",htonl_64(obj.AuctionQty));
         printf("TradingPhaseCode:%s\n",obj.TradingPhaseCode);
         fprintf(logfd,"TradingPhaseCode:%s\n",obj.TradingPhaseCode);
         printf("--------------------------------\n");
         fprintf(logfd,"%s","--------------------------------\n");
         printf("Input 'q' to disconnect API:");
         fflush(stdout);
    }

}

